import logging
import os

from genie.example_filetype_format import FileTypeFormat
from genie import process_functions

logger = logging.getLogger(__name__)


class clinicalSP(FileTypeFormat):

    _fileType = "clinicalSP"

    # VALIDATE FILENAME
    def _validateFilename(self, filePath):
        assert os.path.basename(filePath[0]) == "nonGENIE_data_clinical.txt"

    def _process(self, clinicalSPDf):
        clinicalSPDf["SAMPLE_ID"] = [
            process_functions.checkGenieId(sample, self.center)
            for sample in clinicalSPDf["SAMPLE_ID"]
        ]
        clinicalSPDf["CENTER"] = self.center
        clinicalSPDf["PATIENT_ID"] = [
            process_functions.checkGenieId(sample, self.center)
            for sample in clinicalSPDf["PATIENT_ID"]
        ]
        return clinicalSPDf

    def process_steps(self, clinicalSPDf, newPath, databaseSynId):
        clinicalSPDf = self._process(clinicalSPDf)
        process_functions.updateData(self.syn, databaseSynId, clinicalSPDf, self.center)
        clinicalSPDf.to_csv(newPath, sep="\t", index=False)
        return newPath

    def _validate(self, clinicalDF):
        clinicalDF.columns = [col.upper() for col in clinicalDF.columns]
        # clinicalDF = clinicalDF.fillna("")
        total_error = ""
        warning = ""

        # CHECK: SAMPLE_ID
        haveColumn = process_functions.checkColExist(clinicalDF, "SAMPLE_ID")
        if not haveColumn:
            total_error += (
                "nonGENIE_data_clinical.txt: " "File must have SAMPLE_ID column.\n"
            )
        else:
            if sum(clinicalDF["SAMPLE_ID"].isnull()) > 0:
                total_error += (
                    "nonGENIE_data_clinical.txt: "
                    "There can't be any blank values for SAMPLE_ID\n"
                )

        # CHECK: SEQ_ASSAY_ID
        haveColumn = process_functions.checkColExist(clinicalDF, "SEQ_ASSAY_ID")
        if haveColumn:
            if sum(clinicalDF["SEQ_ASSAY_ID"].isnull()) > 0:
                warning += (
                    "nonGENIE_data_clinical.txt: Please double check "
                    "your SEQ_ASSAY_ID columns, there are empty rows.\n"
                )
        else:
            total_error += (
                "nonGENIE_data_clinical.txt: " "File must have SEQ_ASSAY_ID column.\n"
            )

        # CHECK: PATIENT_ID
        haveColumn = process_functions.checkColExist(clinicalDF, "PATIENT_ID")
        if not haveColumn:
            total_error += (
                "nonGENIE_data_clinical.txt: " "File must have PATIENT_ID column.\n"
            )
        else:
            if sum(clinicalDF["PATIENT_ID"].isnull()) > 0:
                total_error += (
                    "nonGENIE_data_clinical.txt: "
                    "There can't be any blank values for PATIENT_ID\n"
                )
        return (total_error, warning)



from unittest import mock
from unittest.mock import patch

import pandas as pd
import pytest
import synapseclient

from genie_registry.fusions import fusions


def createMockTable(dataframe):
    table = mock.create_autospec(synapseclient.table.CsvFileTable)
    table.asDataFrame.return_value = dataframe
    return table


def table_query_results(*args):
    return table_query_results_map[args]


symbols = pd.DataFrame(
    dict(Hugo_Symbol=["AAK1", "AAED1", "AAAS"], ID=["AAK1", "AAED", "AAAS"])
)

# This is the gene positions that all bed dataframe will be processed against
table_query_results_map = {
    ("select Hugo_Symbol, ID from syn11600834 where CENTER = 'SAGE'",): createMockTable(
        symbols
    ),
}
ENTITY = synapseclient.Project("testing", annotations={"dbMapping": ["syn10967259"]})

syn = mock.create_autospec(synapseclient.Synapse)
syn.tableQuery.side_effect = table_query_results


@pytest.fixture
def fusionClass(genie_config):
    return fusions(syn, "SAGE", genie_config)


def test_processing(fusionClass):
    expectedFusionDf = pd.DataFrame(
        {
            "HUGO_SYMBOL": ["AAED1", "AAK1", "AAAS5"],
            "ENTREZ_GENE_ID": [0, 0, 0],
            "CENTER": ["SAGE", "SAGE", "SAGE"],
            "TUMOR_SAMPLE_BARCODE": [
                "GENIE-SAGE-ID1-1",
                "GENIE-SAGE-ID2-1",
                "GENIE-SAGE-ID1-3",
            ],
            "FUSION": ["AAED1-AAK1", "AAAS-AAK1", "AAAS-AAK1"],
            "DNA_SUPPORT": ["foo", "foo", "foo"],
            "RNA_SUPPORT": ["foo", "foo", "foo"],
            "METHOD": ["foo", "foo", "foo"],
            "FRAME": ["foo", "foo", "foo"],
            "ID": ["AAED", "AAK1", "AAAS5"],
        }
    )

    fusionDf = pd.DataFrame(
        {
            "HUGO_SYMBOL": ["AAED", "AAK1", "AAAS5"],
            "ENTREZ_GENE_ID": [0, 0, float("nan")],
            "CENTER": ["SAGE", "SAGE", "SAGE"],
            "TUMOR_SAMPLE_BARCODE": [
                "GENIE-SAGE-ID1-1",
                "GENIE-SAGE-ID2-1",
                "GENIE-SAGE-ID1-3",
            ],
            "FUSION": ["AAED-AAK1", "AAAS-AAK1", "AAAS-AAK1"],
            "DNA_SUPPORT": ["foo", "foo", "foo"],
            "RNA_SUPPORT": ["foo", "foo", "foo"],
            "METHOD": ["foo", "foo", "foo"],
            "FRAME": ["foo", "foo", "foo"],
        }
    )

    newFusionDf = fusionClass._process(fusionDf)
    assert expectedFusionDf.equals(newFusionDf[expectedFusionDf.columns])


def test_validation__fail_name(fusionClass):
    with pytest.raises(AssertionError):
        fusionClass.validateFilename(["foo"])
    assert fusionClass.validateFilename(["data_fusions_SAGE.txt"]) == "fusions"


def test_validation_perfect(fusionClass):
    fusionDf = pd.DataFrame(
        {
            "HUGO_SYMBOL": ["AAED", "AAK1", "AAAS"],
            "ENTREZ_GENE_ID": [0, 0, 0],
            "CENTER": ["SAGE", "SAGE", "SAGE"],
            "TUMOR_SAMPLE_BARCODE": [
                "GENIE-SAGE-ID1-1",
                "GENIE-SAGE-ID2-1",
                "GENIE-SAGE-ID1-3",
            ],
            "FUSION": ["AAED-AAK1", "AAAS-AAK1", "AAAS-AAK1"],
            "DNA_SUPPORT": ["foo", "foo", "foo"],
            "RNA_SUPPORT": ["foo", "foo", "foo"],
            "METHOD": ["foo", "foo", "foo"],
            "FRAME": ["foo", "foo", "foo"],
        }
    )
    with patch.object(syn, "get", return_value=ENTITY):
        error, warning = fusionClass._validate(fusionDf, False)
        assert error == ""
        assert warning == ""


def test_validation_perfect_invalid(fusionClass):
    fusionDf = pd.DataFrame(
        {
            "HUGO_SYMBOL": [float("nan"), "AAK1", "AAAS"],
            "CENTER": ["SAGE", "SAGE", "SAGE"],
            "TUMOR_SAMPLE_BARCODE": ["ID1-1", "ID2-1", "ID1-3"],
            "FUSION": ["AAED-AAK1", "AAAS-AAK1", "AAAS-AAK1"],
            "DNA_SUPPORT": ["foo", "foo", "foo"],
            "RNA_SUPPORT": ["foo", "foo", "foo"],
            "METHOD": ["foo", "foo", "foo"],
            "FRAME": ["foo", "foo", "foo"],
        }
    )
    with patch.object(syn, "get", return_value=ENTITY):
        error, warning = fusionClass._validate(fusionDf, False)
        expectedErrors = (
            "Your fusion file must at least have these headers: ENTREZ_GENE_ID.\n"
            "Your fusion file should not have any NA/blank Hugo Symbols.\n"
            "fusion: TUMOR_SAMPLE_BARCODE must start with GENIE-SAGE\n"
        )

        assert error == expectedErrors
        assert warning == ""
    # Do not check symbols
    with patch.object(syn, "get", return_value=ENTITY):
        error, warning = fusionClass._validate(fusionDf, True)
        expectedErrors = (
            "Your fusion file must at least have these headers: ENTREZ_GENE_ID.\n"
            "fusion: TUMOR_SAMPLE_BARCODE must start with GENIE-SAGE\n"
        )
        assert error == expectedErrors
        assert warning == ""



from unittest import mock
from unittest.mock import patch
import pytest

import pandas as pd
import synapseclient

from genie_registry.cna import cna


def createMockTable(dataframe):
    table = mock.create_autospec(synapseclient.table.CsvFileTable)
    table.asDataFrame.return_value = dataframe
    return table


def table_query_results(*args):
    return table_query_results_map[args]


database_mapping = pd.DataFrame(dict(Database=["bed"], Id=["syn8457748"]))
symbols = pd.DataFrame(
    dict(
        Hugo_Symbol=["AAK1", "AAED1", "AAAS", "AAED1"],
        ID=["AAK1", "AAED", "AAAS", "AAD"],
    )
)
# This is the gene positions that all bed dataframe will be processed against
table_query_results_map = {
    ("select Hugo_Symbol, ID from syn11600834 where CENTER = 'SAGE'",): createMockTable(
        symbols
    ),
}

syn = mock.create_autospec(synapseclient.Synapse)
syn.tableQuery.side_effect = table_query_results

ENTITY = synapseclient.Project("testing", annotations={"dbMapping": ["syn10967259"]})


@pytest.fixture
def cna_class(genie_config):
    return cna(syn, "SAGE", genie_config)


def test_processing(cna_class):

    order = ["Hugo_Symbol", "Entrez_gene_id", "GENIE-SAGE-Id1-1", "GENIE-SAGE-Id2-1"]

    expected_cnadf = pd.DataFrame(
        {
            "Hugo_Symbol": ["AAED1", "AAK1", "AAAS"],
            "GENIE-SAGE-Id1-1": [-0.5, 2.0, 0.5],
            "GENIE-SAGE-Id2-1": [1.0, 1.5, -1.5],
        }
    )

    cnadf = pd.DataFrame(
        {
            "Hugo_Symbol": ["AAED", "AAK1", "AAAS"],
            "Entrez_gene_id": [0, 0, 0],
            "GENIE-SAGE-Id1-1": [-0.5, 2, 0.5],
            "GENIE-SAGE-Id2-1": [1, 1.5, -1.5],
        }
    )
    cnadf = cnadf[order]
    new_cnadf = cna_class._process(cnadf)
    assert expected_cnadf.equals(new_cnadf[expected_cnadf.columns])

    order = ["Hugo_Symbol", "Entrez_gene_id", "GENIE-SAGE-Id1-1", "GENIE-SAGE-Id2-1"]

    expectedCnaDf = pd.DataFrame(
        {
            "Hugo_Symbol": ["AAAS", "AAED1"],
            "GENIE-SAGE-Id1-1": [float("nan"), 1.0],
            "GENIE-SAGE-Id2-1": [float("nan"), 2.0],
        }
    )

    cnaDf = pd.DataFrame(
        {
            "Hugo_Symbol": ["AAED", "AAED1", "foo", "AAAS", "AAD"],
            "Entrez_gene_id": [0, 0, 0, 0, 0],
            "GENIE-SAGE-Id1-1": [1, 1, 0, float("nan"), 1],
            "GENIE-SAGE-Id2-1": [2, 0, -1, float("nan"), float("nan")],
        }
    )
    cnaDf = cnaDf[order]
    newCnaDf = cna_class._process(cnaDf)
    newCnaDf.reset_index(inplace=True, drop=True)
    pd.testing.assert_frame_equal(expectedCnaDf, newCnaDf[expectedCnaDf.columns])


def test_validation(cna_class):
    with pytest.raises(AssertionError):
        cna_class.validateFilename(["foo"])
    assert cna_class.validateFilename(["data_CNA_SAGE.txt"]) == "cna"

    order = ["Hugo_Symbol", "Entrez_gene_id", "GENIE-SAGE-ID1-1", "GENIE-SAGE-ID2-1"]
    cnaDf = pd.DataFrame(
        {
            "Hugo_Symbol": ["AAED", "AAK1", "AAAS"],
            "Entrez_gene_id": [0, 0, 0],
            "GENIE-SAGE-ID1-1": [-2, 0, 1],
            "GENIE-SAGE-ID2-1": [0.5, 1.5, -1.5],
            "GENIE-SAGE-ID3-1": [float("nan"), 2, "NA"],
        }
    )
    cnaDf = cnaDf[order]
    with patch.object(syn, "get", return_value=ENTITY):
        error, warning = cna_class._validate(cnaDf, False)
        assert error == ""
        assert warning == ""

    cnaDf = pd.DataFrame(
        {
            "Hugo_Symbol": ["foo", "AAED", "AAED1", "AAD"],
            "GENIE-SAGE-ID1-1": [1, 2, float("nan"), 1],
            "GENIE-SAGE-ID2-1": [2, 1, -1, 2],
        }
    )
    cnaDf.sort_values("Hugo_Symbol", inplace=True)
    cnaDf = cnaDf[["GENIE-SAGE-ID1-1", "Hugo_Symbol", "GENIE-SAGE-ID2-1"]]

    with patch.object(syn, "get", return_value=ENTITY):
        error, warning = cna_class._validate(cnaDf, False)
        expectedErrors = (
            "Your cnv file's first column must be Hugo_Symbol\n"
            "Your CNA file has duplicated Hugo_Symbols (After "
            "remapping of genes): AAD,AAED,AAED1 -> AAED1,AAED1,AAED1.\n"
        )

    assert error == expectedErrors
    assert warning == ""
    order = ["Hugo_Symbol", "GENIE-SAGE-ID1-1", "GENIE-SAGE-ID2-1"]

    cnaDf = pd.DataFrame(
        {
            "Hugo_Symbol": ["AAK1", "AAAS", "AAED1"],
            "GENIE-SAGE-ID1-1": [1, 2, 1],
            "GENIE-SAGE-ID2-1": [2, 1, 3],
        }
    )
    cnaDf = cnaDf[order]
    with patch.object(syn, "get", return_value=ENTITY):
        error, warning = cna_class._validate(cnaDf, False)
        expectedErrors = (
            "All values must be NA/blank, -2, -1.5, -1, -0.5, 0, "
            "0.5, 1, 1.5, or 2.\n"
        )
        assert error == expectedErrors
        assert warning == ""



from unittest import mock

import pandas as pd
import pytest
import synapseclient

from genie_registry.sampleRetraction import sampleRetraction
from genie_registry.patientRetraction import patientRetraction

syn = mock.create_autospec(synapseclient.Synapse)

sr = sampleRetraction(syn, "SAGE")
pr = patientRetraction(syn, "SAGE")


def test_processing():

    expectedsrDf = pd.DataFrame(
        dict(
            genieSampleId=[
                "GENIE-SAGE-ID1-1",
                "GENIE-SAGE-ID2-1",
                "GENIE-SAGE-ID3-1",
                "GENIE-SAGE-ID4-1",
                "GENIE-SAGE-ID5-1",
            ],
            retractionDate=[
                1523039400000,
                1523039400000,
                1523039400000,
                1523039400000,
                1523039400000,
            ],
            center=["SAGE", "SAGE", "SAGE", "SAGE", "SAGE"],
        )
    )

    srDf = pd.DataFrame(
        {
            0: [
                "GENIE-SAGE-ID1-1",
                "GENIE-SAGE-ID2-1",
                "GENIE-SAGE-ID3-1",
                "GENIE-SAGE-ID4-1",
                "GENIE-SAGE-ID5-1",
            ]
        }
    )

    newsrDf = sr._process(srDf, "2018-04-06T18:30:00")
    assert expectedsrDf.equals(newsrDf[expectedsrDf.columns])

    expectedprDf = pd.DataFrame(
        dict(
            geniePatientId=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            retractionDate=[
                1523125800000,
                1523125800000,
                1523125800000,
                1523125800000,
                1523125800000,
            ],
            center=["SAGE", "SAGE", "SAGE", "SAGE", "SAGE"],
        )
    )

    prDf = pd.DataFrame(
        {
            0: [
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ]
        }
    )

    newprDf = pr._process(prDf, "2018-04-07T18:30:00")
    assert expectedprDf.equals(newprDf[expectedprDf.columns])


def test_validation():
    with pytest.raises(AssertionError):
        sr.validateFilename(["foo"])
    assert sr.validateFilename(["sampleRetraction.csv"]) == "sampleRetraction"

    with pytest.raises(AssertionError):
        pr.validateFilename(["foo"])
    assert pr.validateFilename(["patientRetraction.csv"]) == "patientRetraction"



"""
Test GENIE filters
"""
import datetime
import os
import sys
from unittest.mock import patch

import pandas as pd

from genie.process_functions import seqDateFilter
from genie import database_to_staging
from genie.database_to_staging import (
    seq_assay_id_filter,
    redact_phi,
    no_genepanel_filter,
    _to_redact_interval,
    _redact_year,
    _to_redact_difference,
)
from genie.consortium_to_public import commonVariantFilter

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(SCRIPT_DIR, "../../genie"))


# syn = synapseclient.login()
def test_seqassayfilter():
    # SEQ ASSAY ID filter, only seq assay ids with more than
    sagetest = ["SAGE-TEST-1"] * 51
    sagetest.extend(["SAGE-TEST-2"] * 10)

    clinicalDf = pd.DataFrame(sagetest)
    clinicalDf.rename(columns={0: "SEQ_ASSAY_ID"}, inplace=True)
    clinicalDf["SAMPLE_ID"] = range(0, len(sagetest))
    # Make sure that only samples of seq assay ids that have 50 or
    # more samples are returned
    samples = seq_assay_id_filter(clinicalDf).reset_index()
    # Must make sure that seq assays have the same index to do comparisons
    del samples["index"]
    samples = samples["SAMPLE_ID"]
    assert all(samples == pd.Series(range(51, 61)))


def test_seqdatefilter():
    # Check that seq date filter works correctly
    SEQ_DATE = "Jan-2018"
    processingDate = datetime.datetime.strptime(SEQ_DATE, "%b-%Y")
    clinicalDf = pd.DataFrame(
        [
            "SAGE-TEST-1",
            "SAGE-TEST-2",
            "SAGE-TEST-3",
            "SAGE-TEST-4",
            "SAGE-TEST-5",
            "SAGE-TEST-6",
            "SAGE-TEST-7",
        ]
    )
    clinicalDf.rename(columns={0: "SAMPLE_ID"}, inplace=True)
    expected = [
        "SAGE-TEST-1",
        "SAGE-TEST-2",
        "SAGE-TEST-3",
        "SAGE-TEST-4",
        "SAGE-TEST-5",
        "SAGE-TEST-6",
    ]
    clinicalDf["SEQ_DATE"] = [
        "Jan-2018",
        "Apr-2018",
        "Oct-2017",
        "Jul-2017",
        "Apr-2017",
        "Jan-2017",
        "Oct-2016",
    ]
    samples = seqDateFilter(clinicalDf, processingDate, 366)
    assert all(samples == expected)

    # Half year filter
    samples = seqDateFilter(clinicalDf, processingDate, 184)
    expected = ["SAGE-TEST-1", "SAGE-TEST-2", "SAGE-TEST-3", "SAGE-TEST-4"]
    assert all(samples == expected)

    # Test leap year
    SEQ_DATE = "Jan-2017"
    processingDate = datetime.datetime.strptime(SEQ_DATE, "%b-%Y")
    clinicalDf = pd.DataFrame(
        [
            "SAGE-TEST-1",
            "SAGE-TEST-2",
            "SAGE-TEST-3",
            "SAGE-TEST-4",
            "SAGE-TEST-5",
            "SAGE-TEST-6",
            "SAGE-TEST-7",
        ]
    )
    clinicalDf.rename(columns={0: "SAMPLE_ID"}, inplace=True)
    clinicalDf["SEQ_DATE"] = [
        "Jan-2017",
        "Apr-2017",
        "Oct-2016",
        "Jul-2016",
        "Apr-2016",
        "Jan-2016",
        "Oct-2015",
    ]
    expected = [
        "SAGE-TEST-1",
        "SAGE-TEST-2",
        "SAGE-TEST-3",
        "SAGE-TEST-4",
        "SAGE-TEST-5",
        "SAGE-TEST-6",
    ]
    samples = seqDateFilter(clinicalDf, processingDate, 366)
    assert all(samples == expected)

    # Check half year release date during leap year
    SEQ_DATE = "Jul-2016"
    processingDate = datetime.datetime.strptime(SEQ_DATE, "%b-%Y")
    samples = seqDateFilter(clinicalDf, processingDate, 184)
    assert all(samples == expected)


def test__to_redact_interval():
    """Tests the correct boolean vectors are returned for phi and pediatric
    redaction"""
    values = pd.Series([32850, 32485, 6570, 6569, "<foo", ">testing"])
    to_redact, to_redact_peds = _to_redact_interval(values)
    expected_redact = [True, False, False, False, False, True]
    expected_redact_peds = [False, False, False, True, True, False]
    assert to_redact.to_list() == expected_redact
    assert to_redact_peds.to_list() == expected_redact_peds


def test__redact_year():
    """Tests redaction of birth year based on < and >"""
    values = pd.Series([1923, 2003, "<foo", ">testing"])
    redacted = _redact_year(values)
    expected_redact = [1923, 2003, "withheld", "cannotReleaseHIPAA"]
    assert redacted.to_list() == expected_redact


def test___to_redact_difference():
    """Tests if a difference between two year columns is >89, redact"""
    year1 = pd.Series([1923, 2000, float("nan")])
    year2 = pd.Series([1926, 2100, 2000])
    redacted = _to_redact_difference(year1, year2)
    expected_redact = [False, True, False]
    assert redacted.to_list() == expected_redact


def test_redact_phi():
    """Redacts PHI interval and years"""
    return_bools = ([True, False, False], [False, False, True])
    clinicaldf = pd.DataFrame(["SAGE-TEST-1", "SAGE-TEST-2", "SAGE-TEST-3"])
    clinicaldf.rename(columns={0: "SAMPLE_ID"}, inplace=True)
    clinicaldf["AGE_AT_SEQ_REPORT"] = [32850, 32485, 6570]
    clinicaldf["BIRTH_YEAR"] = [1900, "<1900", 1902]
    # These are the years that are returned by the _redact_year
    # Use these against YEAR_CONTACT and YEAR_DEATH to calculate
    # expected_birth
    return_year = pd.Series([2000, 1903, 1904])
    clinicaldf["YEAR_CONTACT"] = [2100, 1904, float("nan")]
    clinicaldf["YEAR_DEATH"] = [2001, float("nan"), 2000]

    expected_age = pd.Series([">32485", 32485, "<6570"])
    expected_birth = pd.Series(["cannotReleaseHIPAA", 1903, "cannotReleaseHIPAA"])
    with patch.object(
        database_to_staging, "_to_redact_interval", return_value=return_bools
    ), patch.object(database_to_staging, "_redact_year", return_value=return_year):
        redacted_clin = redact_phi(
            clinicaldf, interval_cols_to_redact=["AGE_AT_SEQ_REPORT"]
        )
        assert all(redacted_clin["AGE_AT_SEQ_REPORT"] == expected_age)
        assert all(redacted_clin["BIRTH_YEAR"] == expected_birth)


# def test_MAFinBED():
#   syn = mock.create_autospec(synapseclient.Synapse)
#   # MAF in BED filter (Make sure that the only Hugo symbol that
# passes through this filter is the ERRFI1 gene)
#   databaseSynIdMapping = syn.tableQuery('select * from syn11600968')
#   databaseSynIdMappingDf = databaseSynIdMapping.asDataFrame()
# CENTER_MAPPING = syn.tableQuery(
#     'SELECT * FROM syn11601248 where stagingSynId is not null')
#   CENTER_MAPPING_DF = CENTER_MAPPING.asDataFrame()
#   mafPath = os.path.join(SCRIPT_DIR,"testingmaf.txt")
# temp = runMAFinBED(
#     syn, mafPath, CENTER_MAPPING_DF, databaseSynIdMappingDf, test=True)
#   maf = pd.read_csv(mafPath,sep="\t")
#   assert all(maf['Hugo_Symbol'][maf['inBED'] ==  True].unique() == "ERRFI1")
#   os.unlink(mafPath)

# def test_MutationInCis():
#   def createMockTable(dataframe):
#       table = mock.create_autospec(synapseclient.table.CsvFileTable)
#       table.asDataFrame.return_value= dataframe
#       return(table)

#   def table_query_results(*args):
#       return(table_query_results_map[args])

#   databaseMapping = pd.DataFrame(dict(Database=['mutationsInCis'],
#                                       Id=['syn7765462']))

#   centerMapping = pd.DataFrame(dict(center=['SAGE'],
#                                     inputSynId=['syn11601335'],
#                                     stagingSynId=['syn11601337']))

#   mutCisDf = pd.DataFrame(dict(Flag=['mutationsInCis'],
#                                  Center=['syn7765462'],
#                                  Tumor_Sample_Barcode=["GENIE-SAGE-ID1-1"],
#                                  Variant_Classification=["Nonsense_Mutation"],
#                                  Hugo_Symbol=["AKT1"],
#                                  HGVSp_Short=["p.1234"],
#                                  Chromosome=["1"],
#                                  Start_Position=[324234],
#                                  Reference_Allele=["AGC"],
#                                  Tumor_Seq_Allele2=["GCCCT"],
#                                  t_alt_count_num=[3],
#                                  t_depth=[234]))

#   #This is the gene positions that all bed
#   dataframe will be processed against
# table_query_results_map = {
#     ('select * from syn11600968',):
#         createMockTable(databaseMapping),
#     ('SELECT * FROM syn11601248 where staginsgSynId is not null',):
#         createMockTable(centerMapping),
#     ("select * from syn7765462 where Center = 'SAGE'",):
#         createMockTable(mutCisDf)
# }

#   syn = mock.create_autospec(synapseclient.Synapse)
#   syn.tableQuery.side_effect=table_query_results
#   # Mutation in cis filter check (There is one TOSS sample,
#   it should be GENIE-TEST-0-1)
# remove_samples = mutation_in_cis_filter(
#     syn, True, None, None, "syn11601206", None)
#   assert  all(remove_samples == "GENIE-TEST-0-1")


def test_commonvariantfilter():
    """
    Test common variant filter, make sure none
    of the common_variant rows are kept
    """
    mutationDf = pd.DataFrame(
        [
            "common_variant",
            "test1",
            "asdfasd:common_variant",
            "fo;common_variant",
            "test2",
            "test3",
        ]
    )
    mutationDf.rename(columns={0: "FILTER"}, inplace=True)
    maf = commonVariantFilter(mutationDf)
    expected = ["test1", "test2", "test3"]
    assert all(maf["FILTER"] == expected)


def test_no_genepanel_filter():
    """
    Tests the filter to remove samples
    with no bed files
    """
    sagetest = ["SAGE-TEST-1"] * 51
    beddf = pd.DataFrame(sagetest)
    beddf.rename(columns={0: "SEQ_ASSAY_ID"}, inplace=True)
    sagetest.extend(["SAGE-TEST-2"] * 10)

    clinicaldf = pd.DataFrame(sagetest)
    clinicaldf.rename(columns={0: "SEQ_ASSAY_ID"}, inplace=True)
    clinicaldf["SAMPLE_ID"] = sagetest

    remove_samples = no_genepanel_filter(clinicaldf, beddf)
    assert all(remove_samples == ["SAGE-TEST-2"] * 10)



import logging
import os

import synapseclient

from genie.example_filetype_format import FileTypeFormat

logger = logging.getLogger(__name__)


class workflow(FileTypeFormat):

    _fileType = "md"

    _process_kwargs = ["databaseSynId"]

    def _validateFilename(self, filePath):
        assert os.path.basename(filePath[0]).startswith(
            self.center
        ) and os.path.basename(filePath[0]).endswith(".md")

    def process_steps(self, filePath, databaseSynId):
        self.syn.store(synapseclient.File(filePath, parent=databaseSynId))
        return filePath



import os
import logging

import pandas as pd

from genie.example_filetype_format import FileTypeFormat
from genie import process_functions

logger = logging.getLogger(__name__)


def validateSymbol(x, bedDf, returnMappedDf=True):
    valid = False
    gene = x["HUGO_SYMBOL"]
    if sum(bedDf["Hugo_Symbol"] == gene) > 0:
        valid = True
    elif sum(bedDf["ID"] == gene) > 0:
        mismatch = bedDf[bedDf["ID"] == gene]
        mismatch.drop_duplicates(inplace=True)
        logger.info(
            "%s will be remapped to %s" % (gene, mismatch["Hugo_Symbol"].values[0])
        )
        x["HUGO_SYMBOL"] = mismatch["Hugo_Symbol"].values[0]
    # else:
    #    logger.warning("%s cannot be remapped and will not be released. The symbol must exist in your seq assay ids (bed files) and must be mappable to a gene." % gene)
    #    x['HUGO_SYMBOL'] = float('nan')
    # x['FUSION'] = x['FUSION'].replace("%s-" % gene,"%s-" % x['HUGO_SYMBOL'])
    # x['COMMENTS'] = str(x['COMMENTS']).replace("-%s" % gene,"-%s" % str(x['COMMENTS']))
    if returnMappedDf:
        return x
    else:
        return valid


# Remap fusion's FUSION column
def remapFusion(gene_dict, DF, col):
    nonmapped = []
    total = []
    for each in DF[col]:
        for key in gene_dict:
            value = gene_dict[key]
            if value == False:
                nonmapped.append(key)
            else:
                each = each.replace("%s-" % key, "%s-" % value)
                each = each.replace("-%s fusion" % key, "-%s fusion" % value)
        total.append(each)
    DF[col] = total
    return (DF, nonmapped)


class fusions(FileTypeFormat):

    _fileType = "fusions"

    _process_kwargs = ["newPath", "databaseSynId"]

    _validation_kwargs = ["nosymbol_check"]

    # VALIDATE FILENAME
    def _validateFilename(self, filePath):
        assert os.path.basename(filePath[0]) == "data_fusions_%s.txt" % self.center

    def _process(self, fusion):
        fusion.columns = [col.upper() for col in fusion.columns]
        fusion["CENTER"] = self.center

        # This is temporary, because comments column will be removed
        # if fusion.get("COMMENTS") is None:
        #    fusion['COMMENTS'] = ""
        # #Will remove comments column
        # fusion['COMMENTS'] = ""
        fusion["ENTREZ_GENE_ID"] = fusion["ENTREZ_GENE_ID"].fillna(0)
        fusion = fusion.drop_duplicates()
        fusion["ID"] = fusion["HUGO_SYMBOL"].copy()
        bedSynId = self.genie_config["bed"]
        bed = self.syn.tableQuery(
            f"select Hugo_Symbol, ID from {bedSynId} where CENTER = '{self.center}'"
        )
        bedDf = bed.asDataFrame()
        fusion = fusion.apply(lambda x: validateSymbol(x, bedDf), axis=1)
        # Create nonmapped gene dict
        temp = fusion[fusion["HUGO_SYMBOL"] != fusion["ID"]]
        foo = temp[~temp.HUGO_SYMBOL.isnull()]
        temp = foo[["HUGO_SYMBOL", "ID"]]
        temp.drop_duplicates(inplace=True)
        temp.index = temp.ID
        del temp["ID"]
        # fusion = fusion[~fusion['HUGO_SYMBOL'].isnull()]
        fusion["FUSION"] = fusion["FUSION"].fillna("")
        fusion, nonmapped = remapFusion(temp.to_dict()["HUGO_SYMBOL"], fusion, "FUSION")
        # Fill in blank hugo symbol columns with original symbol
        null_symbols_idx = fusion["HUGO_SYMBOL"].isnull()
        fusion["HUGO_SYMBOL"][null_symbols_idx] = fusion["ID"][null_symbols_idx]
        # fusion, nonmapped = remapFusion(temp.to_dict()['HUGO_SYMBOL'], fusion, "COMMENTS")
        fusion["ENTREZ_GENE_ID"] = [int(float(i)) for i in fusion["ENTREZ_GENE_ID"]]
        return fusion

    # PROCESSING
    def process_steps(self, fusion, databaseSynId, newPath):
        fusion = self._process(fusion)
        process_functions.updateData(
            self.syn, databaseSynId, fusion, self.center, toDelete=True
        )
        fusion.to_csv(newPath, sep="\t", index=False)
        return newPath

    def _validate(self, fusionDF, nosymbol_check):
        total_error = ""
        warning = ""

        # Frame: "in-frame" or "frameshift".
        # Fusion_Status (OPTIONAL): An assessment of the mutation type (i.e., "SOMATIC", "GERMLINE", "UNKNOWN", or empty)

        fusionDF.columns = [col.upper() for col in fusionDF.columns]

        REQUIRED_HEADERS = pd.Series(
            [
                "HUGO_SYMBOL",
                "ENTREZ_GENE_ID",
                "CENTER",
                "TUMOR_SAMPLE_BARCODE",
                "FUSION",
                "DNA_SUPPORT",
                "RNA_SUPPORT",
                "METHOD",
                "FRAME",
            ]
        )
        if fusionDF.get("COMMENTS") is None:
            fusionDF["COMMENTS"] = float("nan")
        if not all(REQUIRED_HEADERS.isin(fusionDF.columns)):
            total_error += (
                "Your fusion file must at least have these headers: %s.\n"
                % ",".join(REQUIRED_HEADERS[~REQUIRED_HEADERS.isin(fusionDF.columns)])
            )
        if (
            process_functions.checkColExist(fusionDF, "HUGO_SYMBOL")
            and not nosymbol_check
        ):
            # logger.info("VALIDATING %s GENE SYMBOLS" % os.path.basename(filePath))
            # invalidated_genes = fusionDF["HUGO_SYMBOL"].drop_duplicates().apply(validateSymbol)
            # bedSynId = self.genie_config['bed']
            # bed = self.syn.tableQuery(
            #     f"select Hugo_Symbol, ID from {bedSynId} where CENTER = '{self.center}'"
            # )
            # bedDf = bed.asDataFrame()
            # invalidated_genes = self.pool.map(process_functions.validateSymbol, fusionDF["HUGO_SYMBOL"].drop_duplicates())
            if fusionDF["HUGO_SYMBOL"].isnull().any():
                total_error += (
                    "Your fusion file should not have any NA/blank Hugo Symbols.\n"
                )
            # fusionDF = fusionDF.drop_duplicates("HUGO_SYMBOL").apply(lambda x: validateSymbol(x, bedDf), axis=1)

        if process_functions.checkColExist(fusionDF, "TUMOR_SAMPLE_BARCODE"):
            error = process_functions.validate_genie_identifier(
                identifiers=fusionDF["TUMOR_SAMPLE_BARCODE"],
                center=self.center,
                filename="fusion",
                col="TUMOR_SAMPLE_BARCODE",
            )
            total_error += error
        # if process_functions.checkColExist(fusionDF, "DNA_SUPPORT"):
        #     if not fusionDF.DNA_SUPPORT.isin(["yes","no","unknown"]).all():
        #         total_error += "Your fusion file's DNA_SUPPORT column must be 'yes', 'no', or 'unknown'"

        # if process_functions.checkColExist(fusionDF, "RNA_SUPPORT"):
        #     if not fusionDF.RNA_SUPPORT.isin(["yes","no","unknown"]).all():
        #         total_error += "Your fusion file's RNA_SUPPORT column must be 'yes', 'no', or 'unknown'"

        # if process_functions.checkColExist(fusionDF, "FRAME"):
        #     if not fusionDF.FRAME.isin(["in-frame","frameshift"]).all():
        #         total_error += "Your fusion file's FRAME column must be 'in-frame', or 'frameshift'"

        return (total_error, warning)



import datetime
import json
from unittest import mock

import pandas as pd
import pytest
import synapseclient

import genie_registry
from genie_registry.clinical import Clinical


def createMockTable(dataframe):
    table = mock.create_autospec(synapseclient.table.CsvFileTable)
    table.asDataFrame.return_value = dataframe
    return table


def table_query_results(*args):
    return table_query_results_map[args]


no_nan = pd.DataFrame(
    dict(
        CODE=[1, 2, 3, 4, 99],
        CBIO_LABEL=["Test", "Why", "foo", "Me", "Unknown"],
        DESCRIPTION=["non", "asdf", "asdf", "asdff", "asdfasdf"],
    )
)

sexdf = pd.DataFrame(
    dict(
        CODE=[1, 2, 99],
        CBIO_LABEL=["Male", "Female", "Unknown"],
        DESCRIPTION=["Male", "Female", "Not coded"],
    )
)

table_query_results_map = {
    ("select * from syn7434222",): createMockTable(sexdf),
    ("select * from syn7434236",): createMockTable(no_nan),
    ("select * from syn7434242",): createMockTable(no_nan),
    ("select * from syn7434273",): createMockTable(no_nan),
}

syn = mock.create_autospec(synapseclient.Synapse)
syn.tableQuery.side_effect = table_query_results

json_oncotreeurl = (
    "http://oncotree.mskcc.org/api/tumorTypes/tree?version=oncotree_2017_06_21"
)

onco_map_dict = {
    "AMPCA": {
        "CANCER_TYPE": "Ampullary Cancer",
        "CANCER_TYPE_DETAILED": "Ampullary Carcinoma",
        "ONCOTREE_PRIMARY_NODE": "AMPULLA_OF_VATER",
        "ONCOTREE_SECONDARY_NODE": "AMPCA",
    },
    "TESTIS": {
        "CANCER_TYPE": "Testicular Cancer, NOS",
        "CANCER_TYPE_DETAILED": "Testis",
        "ONCOTREE_PRIMARY_NODE": "TESTIS",
        "ONCOTREE_SECONDARY_NODE": "",
    },
    "UCEC": {
        "CANCER_TYPE": "Endometrial Cancer",
        "CANCER_TYPE_DETAILED": "Endometrial Carcinoma",
        "ONCOTREE_PRIMARY_NODE": "UTERUS",
        "ONCOTREE_SECONDARY_NODE": "UCEC",
    },
}


@pytest.fixture
def clin_class(genie_config):
    return Clinical(syn, "SAGE", genie_config=genie_config)


def test_filetype(clin_class):
    assert clin_class._fileType == "clinical"


@pytest.fixture(params=[(["foo"]), (["foo", "data_clinical_supp_sample_SAGE.txt"])])
def filename_fileformat_map(clin_class, request):
    return request.param


def test_incorrect_validatefilename(clin_class, filename_fileformat_map):
    filepath_list = filename_fileformat_map
    with pytest.raises(AssertionError):
        clin_class.validateFilename(filepath_list)


def test_correct_validatefilename(clin_class):
    assert clin_class.validateFilename(["data_clinical_supp_SAGE.txt"]) == "clinical"
    assert (
        clin_class.validateFilename(
            [
                "data_clinical_supp_sample_SAGE.txt",
                "data_clinical_supp_patient_SAGE.txt",
            ]
        )
        == "clinical"
    )


def test_patient_fillvs__process(clin_class):
    """
    This will be removed once vital status values are required
    - capitalized column headers
    - remapping of values
    - Fill out CENTER column
    - Append GENIE-CENTER-..
    """
    expected_patientdf = pd.DataFrame(
        dict(
            PATIENT_ID=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            SEX=["Male", "Female", "Male", "Female", "Unknown"],
            PRIMARY_RACE=["Test", "Why", "foo", "Me", "Unknown"],
            SECONDARY_RACE=["Test", "Why", "foo", "Me", "Unknown"],
            TERTIARY_RACE=["Test", "Why", "foo", "Me", "Unknown"],
            ETHNICITY=["Test", "Why", "foo", "Me", "Unknown"],
            BIRTH_YEAR=[1990, 1990, 1990, 1990, 1990],
            CENTER=["SAGE", "SAGE", "SAGE", "SAGE", "SAGE"],
        )
    )

    patientdf = pd.DataFrame(
        dict(
            PATIENT_Id=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            sex=[1, 2, 1, 2, 99],
            PRIMARY_RACE=[1, 2, 3, 4, 99],
            Secondary_RACE=[1, 2, 3, 4, 99],
            TERTIARY_RACE=[1, 2, 3, 4, 99],
            ETHNICITY=[1, 2, 3, 4, 99],
            BIRTH_YEAR=[1990, 1990, 1990, 1990, 1990],
            CENTER=["FOO", "FOO", "FOO", "FOO", "FOO"],
        )
    )
    patient_cols = [
        "PATIENT_ID",
        "SEX",
        "PRIMARY_RACE",
        "SECONDARY_RACE",
        "TERTIARY_RACE",
        "ETHNICITY",
        "BIRTH_YEAR",
        "CENTER",
        "YEAR_CONTACT",
        "YEAR_DEATH",
        "INT_CONTACT",
        "INT_DOD",
        "DEAD",
    ]
    clinical_template = pd.DataFrame(columns=patient_cols)
    new_patientdf = clin_class._process(patientdf, clinical_template)
    assert new_patientdf.columns.isin(expected_patientdf.columns).all()
    assert expected_patientdf.equals(new_patientdf[expected_patientdf.columns])


def test_patient_lesscoltemplate__process(clin_class):
    """
    Test scope is excluding values.
    Only those value defined by the scope will be written out
    """
    expected_patientdf = pd.DataFrame(
        dict(
            PATIENT_ID=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            SEX=["Male", "Female", "Male", "Female", "Unknown"],
            PRIMARY_RACE=["Test", "Why", "foo", "Me", "Unknown"],
            SECONDARY_RACE=["Test", "Why", "foo", "Me", "Unknown"],
            TERTIARY_RACE=["Test", "Why", "foo", "Me", "Unknown"],
            ETHNICITY=["Test", "Why", "foo", "Me", "Unknown"],
            BIRTH_YEAR=[1990, 1990, 1990, 1990, 1990],
            CENTER=["SAGE", "SAGE", "SAGE", "SAGE", "SAGE"],
        )
    )
    # TEST patient processing
    patientdf = pd.DataFrame(
        dict(
            PATIENT_Id=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            sex=[1, 2, 1, 2, 99],
            PRIMARY_RACE=[1, 2, 3, 4, 99],
            Secondary_RACE=[1, 2, 3, 4, 99],
            TERTIARY_RACE=[1, 2, 3, 4, 99],
            ETHNICITY=[1, 2, 3, 4, 99],
            BIRTH_YEAR=[1990, 1990, 1990, 1990, 1990],
            CENTER=["FOO", "FOO", "FOO", "FOO", "FOO"],
        )
    )
    patient_cols = [
        "PATIENT_ID",
        "SEX",
        "PRIMARY_RACE",
        "SECONDARY_RACE",
        "TERTIARY_RACE",
        "ETHNICITY",
        "BIRTH_YEAR",
        "CENTER",
        "YEAR_CONTACT",
        "YEAR_DEATH",
    ]
    clinical_template = pd.DataFrame(columns=patient_cols)
    new_patientdf = clin_class._process(patientdf, clinical_template)

    assert new_patientdf.columns.isin(patient_cols).all()
    assert expected_patientdf[expected_patientdf.columns].equals(
        new_patientdf[expected_patientdf.columns]
    )


def test_patient_vs__process(clin_class):
    """
    Test vital status columns being propogated with same data
    """
    expected_patientdf = pd.DataFrame(
        dict(
            PATIENT_ID=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            SEX=["Male", "Female", "Male", "Female", "Unknown"],
            PRIMARY_RACE=["Test", "Why", "foo", "Me", "Unknown"],
            SECONDARY_RACE=["Test", "Why", "foo", "Me", "Unknown"],
            TERTIARY_RACE=["Test", "Why", "foo", "Me", "Unknown"],
            ETHNICITY=["Test", "Why", "foo", "Me", "Unknown"],
            BIRTH_YEAR=[1990, 1990, 1990, 1990, 1990],
            CENTER=["SAGE", "SAGE", "SAGE", "SAGE", "SAGE"],
            YEAR_DEATH=["Unknown", "Not Collected", "Not Applicable", 1990, 1990],
            YEAR_CONTACT=["Unknown", "Not Collected", 1990, 1990, 1990],
            INT_CONTACT=[">32485", "<6570", "Unknown", "Not Collected", 2000],
            INT_DOD=[">32485", "<6570", "Unknown", "Not Collected", "Not Applicable"],
            DEAD=[True, False, "Unknown", "Not Collected", True],
        )
    )
    # TEST patient processing
    # Clinical file headers are capitalized prior to processing
    patientdf = pd.DataFrame(
        dict(
            PATIENT_Id=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            sex=[1, 2, 1, 2, 99],
            PRIMARY_RACE=[1, 2, 3, 4, 99],
            Secondary_RACE=[1, 2, 3, 4, 99],
            TERTIARY_RACE=[1, 2, 3, 4, 99],
            ETHNICITY=[1, 2, 3, 4, 99],
            BIRTH_YEAR=[1990, 1990, 1990, 1990, 1990],
            CENTER=["FOO", "FOO", "FOO", "FOO", "FOO"],
            YEAR_DEATH=["Unknown", "Not Collected", "Not Applicable", 1990, 1990],
            YEAR_CONTACT=["Unknown", "Not Collected", 1990, 1990, 1990],
            INT_CONTACT=[">32485", "<6570", "Unknown", "Not Collected", 2000],
            INT_DOD=[">32485", "<6570", "Unknown", "Not Collected", "Not Applicable"],
            DEAD=[True, False, "Unknown", "Not Collected", True],
        )
    )
    patient_cols = [
        "PATIENT_ID",
        "SEX",
        "PRIMARY_RACE",
        "SECONDARY_RACE",
        "TERTIARY_RACE",
        "ETHNICITY",
        "BIRTH_YEAR",
        "CENTER",
        "YEAR_CONTACT",
        "YEAR_DEATH",
        "INT_CONTACT",
        "INT_DOD",
        "DEAD",
    ]
    clinical_template = pd.DataFrame(columns=patient_cols)
    new_patientdf = clin_class._process(patientdf, clinical_template)
    assert expected_patientdf.equals(new_patientdf[expected_patientdf.columns])


def test_sample__process(clin_class):
    """
    Test sample processing
    - column headers are capitalized
    - SEQ_DATE is normalized (Mon-YYYY, Release)
    - Allow UNKNOWN oncotree value
    - Add on GENIE-CENTER-...
    - Remapping of SAMPLE_TYPE/SAMPLE_TYPE_DETAILED value
    - SEQ_YEAR from SEQ_DATE, nan if SEQ_DATE is Release

    """
    expected_sampledf = pd.DataFrame(
        dict(
            SAMPLE_ID=[
                "GENIE-SAGE-ID1-1",
                "GENIE-SAGE-ID2-1",
                "GENIE-SAGE-ID3-1",
                "GENIE-SAGE-ID4-1",
                "GENIE-SAGE-ID5-1",
            ],
            PATIENT_ID=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            AGE_AT_SEQ_REPORT=[100000, 100000, 100000, 100000, 100000],
            ONCOTREE_CODE=["AMPCA", "UNKNOWN", "AMPCA", "AMPCA", "AMPCA"],
            SAMPLE_TYPE=["Test", "Why", "foo", "Me", "Me"],
            CENTER=["SAGE", "SAGE", "SAGE", "SAGE", "SAGE"],
            SAMPLE_TYPE_DETAILED=["non", "asdf", "asdf", "asdff", "asdff"],
            SEQ_ASSAY_ID=["SAGE-1", "SAGE-1", "SAGE-1", "SAGE-1", "SAGE-1"],
            SEQ_DATE=["Jan-2012", "Apr-2013", "Jul-2014", "Oct-2015", "Release"],
            SEQ_YEAR=[2012, 2013, 2014, 2015, float("nan")],
        )
    )
    sample_cols = [
        "SAMPLE_ID",
        "PATIENT_ID",
        "AGE_AT_SEQ_REPORT",
        "ONCOTREE_CODE",
        "SAMPLE_TYPE",
        "SEQ_ASSAY_ID",
        "SEQ_DATE",
        "SAMPLE_TYPE_DETAILED",
        "SEQ_YEAR",
    ]

    clinical_template = pd.DataFrame(columns=sample_cols)
    # patient = False
    sampledf = pd.DataFrame(
        dict(
            SAMPLE_ID=[
                "GENIE-SAGE-ID1-1",
                "GENIE-SAGE-ID2-1",
                "GENIE-SAGE-ID3-1",
                "GENIE-SAGE-ID4-1",
                "GENIE-SAGE-ID5-1",
            ],
            PATIENT_ID=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            Age_AT_SEQ_REPORT=[100000, 100000, 100000, 100000, 100000],
            ONCOTree_CODE=["AMPCA", " UNKNOWN", "AMPCA", "AMPCA", "AMPCA"],
            SAMPLE_TYPE=[1, 2, 3, 4, 4],
            SEQ_ASSAY_ID=["SAGE-1", "SAGE-1", "SAGE-1", "SAGE-1", "SAGE-1"],
            SEQ_DATE=["Jan-2012", "Apr-2013", "JUL-2014", "Oct-2015", "release"],
        )
    )

    new_sampledf = clin_class._process(sampledf, clinical_template)
    assert new_sampledf.columns.isin(expected_sampledf.columns).all()
    assert expected_sampledf.equals(new_sampledf[expected_sampledf.columns])


def test_perfect__validate(clin_class):
    """
    Test perfect validation
    """
    patientdf = pd.DataFrame(
        dict(
            PATIENT_ID=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            SEX=[1, 2, 1, 2, 99],
            PRIMARY_RACE=[1, 2, 3, 4, 99],
            SECONDARY_RACE=[1, 2, 3, 4, 99],
            TERTIARY_RACE=[1, 2, 3, 4, 99],
            ETHNICITY=[1, 2, 3, 4, 99],
            BIRTH_YEAR=[1222, "Unknown", 1920, 1990, 1990],
            CENTER=["FOO", "FOO", "FOO", "FOO", "FOO"],
            YEAR_CONTACT=["Unknown", "Not Collected", ">89", "<18", 1990],
            INT_CONTACT=["Unknown", "Not Collected", ">32485", "<6570", 2000],
            YEAR_DEATH=["Unknown", "Not Collected", "Unknown", "Not Applicable", "<18"],
            INT_DOD=["Unknown", "Not Collected", "Unknown", "Not Applicable", "<6570"],
            DEAD=["Unknown", "Not Collected", "Unknown", False, True],
        )
    )

    sampledf = pd.DataFrame(
        dict(
            SAMPLE_ID=[
                "GENIE-SAGE-ID1-1",
                "GENIE-SAGE-ID2-1",
                "GENIE-SAGE-ID3-1",
                "GENIE-SAGE-ID4-1",
                "GENIE-SAGE-ID5-1",
            ],
            PATIENT_ID=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            AGE_AT_SEQ_REPORT=[">32485", "Unknown", "<6570", 20000, 100000],
            ONCOTREE_CODE=["AMPCA", "AMPCA", "Unknown", "AMPCA", "AMPCA"],
            SAMPLE_TYPE=[1, 2, 3, 4, 4],
            SEQ_ASSAY_ID=["SAGE-1-1", "SAGE-SAGE-1", "SAGE-1", "SAGE-1", "SAGE-1"],
            SEQ_DATE=["Jan-2013", "ApR-2013", "Jul-2013", "Oct-2013", "release"],
            SAMPLE_CLASS=["Tumor", "cfDNA", "cfDNA", "cfDNA", "cfDNA"],
        )
    )

    clinicaldf = patientdf.merge(sampledf, on="PATIENT_ID")
    with mock.patch(
        "genie.process_functions.get_oncotree_code_mappings", return_value=onco_map_dict
    ) as mock_get_onco_map:
        error, warning = clin_class._validate(clinicaldf)
        mock_get_onco_map.called_once_with(json_oncotreeurl)
        assert error == ""
        assert warning == ""


def test_nonull__validate(clin_class):
    """
    Test that no null values are allowed in the clinical dataframe
    """
    patientdf = pd.DataFrame(
        dict(
            PATIENT_ID=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            SEX=[1, 2, 1, 2, float("nan")],
            PRIMARY_RACE=[1, 2, 3, 4, float("nan")],
            SECONDARY_RACE=[1, 2, 3, 4, float("nan")],
            TERTIARY_RACE=[1, 2, 3, 4, float("nan")],
            ETHNICITY=[1, 2, 3, 4, float("nan")],
            BIRTH_YEAR=[float("nan"), "Unknown", 1920, 1990, 1990],
            CENTER=["FOO", "FOO", "FOO", "FOO", "FOO"],
            YEAR_DEATH=[
                "Unknown",
                "Not Collected",
                "Not Applicable",
                1990,
                float("nan"),
            ],
            YEAR_CONTACT=["Unknown", "Not Collected", float("nan"), 1990, 1990],
            INT_CONTACT=["Unknown", "Not Collected", ">32485", float("nan"), 2000],
            INT_DOD=["Unknown", "Not Collected", "Unknown", float("nan"), "<6570"],
            DEAD=["Unknown", "Not Collected", "Unknown", float("nan"), True],
        )
    )

    sampledf = pd.DataFrame(
        dict(
            SAMPLE_ID=[
                "GENIE-SAGE-ID1-1",
                "GENIE-SAGE-ID2-1",
                "GENIE-SAGE-ID3-1",
                "GENIE-SAGE-ID4-1",
                "GENIE-SAGE-ID5-1",
            ],
            PATIENT_ID=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                "GENIE-SAGE-ID5",
            ],
            AGE_AT_SEQ_REPORT=[100000, "Unknown", 20000, float("nan"), 100000],
            ONCOTREE_CODE=["AMPCA", "AMPCA", "Unknown", "AMPCA", "AMPCA"],
            SAMPLE_TYPE=[1, 2, 3, 4, float("nan")],
            SEQ_ASSAY_ID=["SAGE-1-1", "SAGE-SAGE-1", "SAGE-1", "SAGE-1", "SAGE-1"],
            SEQ_DATE=["Jan-2013", "ApR-2013", "Jul-2013", "Oct-2013", "release"],
            SAMPLE_CLASS=["Tumor", "cfDNA", "cfDNA", "cfDNA", float("nan")],
        )
    )

    clinicaldf = patientdf.merge(sampledf, on="PATIENT_ID")
    with mock.patch(
        "genie.process_functions.get_oncotree_code_mappings", return_value=onco_map_dict
    ) as mock_get_onco_map:
        error, warning = clin_class._validate(clinicaldf)
        mock_get_onco_map.called_once_with(json_oncotreeurl)
        expected_errors = (
            "Sample Clinical File: Please double check your "
            "AGE_AT_SEQ_REPORT. It must be an integer, 'Unknown', "
            "'>32485', '<6570'.\n"
            "Sample Clinical File: Please double check your SAMPLE_TYPE "
            "column.  This column must only be these values: 1, 2, 3, 4, 99\n"
            "Patient Clinical File: Please double check your BIRTH_YEAR "
            "column, it must be an integer in YYYY format <= {year} or "
            "'Unknown', '>89', '<18'.\n"
            "Patient Clinical File: Please double check your YEAR_DEATH "
            "column, it must be an integer in YYYY format <= {year} or "
            "'Unknown', 'Not Collected', 'Not Applicable', "
            "'Not Released', '>89', '<18'.\n"
            "Patient Clinical File: Please double check your YEAR_CONTACT "
            "column, it must be an integer in YYYY format <= {year} or "
            "'Unknown', 'Not Collected', 'Not Released', '>89', '<18'.\n"
            "Patient Clinical File: Please double check your INT_CONTACT "
            "column, it must be an integer, '>32485', '<6570', 'Unknown', "
            "'Not Released' or 'Not Collected'.\n"
            "Patient Clinical File: Please double check your INT_DOD "
            "column, it must be an integer, '>32485', '<6570', 'Unknown', "
            "'Not Collected', 'Not Released' or 'Not Applicable'.\n"
            "Patient Clinical File: Please double check your DEAD column, "
            "it must be True, False, 'Unknown', "
            "'Not Released' or 'Not Collected'.\n"
            "Patient: you have inconsistent redaction values in "
            "YEAR_CONTACT, INT_CONTACT.\n"
            "Patient: you have inconsistent redaction and text values in "
            "YEAR_DEATH, INT_DOD.\n"
            "Sample Clinical File: SAMPLE_CLASS column must be 'Tumor', or 'cfDNA'\n"
            "Patient Clinical File: Please double check your PRIMARY_RACE "
            "column.  This column must only be these values: 1, 2, 3, 4, 99\n"
            "Patient Clinical File: Please double check your SECONDARY_RACE "
            "column.  This column must only be these values: 1, 2, 3, 4, 99\n"
            "Patient Clinical File: Please double check your TERTIARY_RACE "
            "column.  This column must only be these values: 1, 2, 3, 4, 99\n"
            "Patient Clinical File: Please double check your SEX column.  "
            "This column must only be these values: 1, 2, 99\n"
            "Patient Clinical File: Please double check your ETHNICITY "
            "column.  This column must only be these values: 1, 2, 3, 4, 99\n"
        )
        assert error == expected_errors.format(year=datetime.datetime.utcnow().year)
        assert warning == ""


def test_missingcols__validate(clin_class):
    """
    Test for missing column errors
    """
    clinicaldf = pd.DataFrame()
    with mock.patch(
        "genie.process_functions.get_oncotree_code_mappings", return_value=onco_map_dict
    ) as mock_get_onco_map:
        error, warning = clin_class._validate(clinicaldf)
        mock_get_onco_map.called_once_with(json_oncotreeurl)
        expected_errors = (
            "Sample Clinical File: Must have SAMPLE_ID column.\n"
            "Patient Clinical File: Must have PATIENT_ID column.\n"
            "Sample Clinical File: Must have AGE_AT_SEQ_REPORT column.\n"
            "Sample Clinical File: Must have ONCOTREE_CODE column.\n"
            "Sample Clinical File: Must have SAMPLE_TYPE column.\n"
            "Sample Clinical File: Must have SEQ_ASSAY_ID column.\n"
            "Sample Clinical File: Must have SEQ_DATE column.\n"
            "Patient Clinical File: Must have BIRTH_YEAR column.\n"
            "Patient Clinical File: Must have YEAR_DEATH column.\n"
            "Patient Clinical File: Must have YEAR_CONTACT column.\n"
            "Patient Clinical File: Must have INT_CONTACT column.\n"
            "Patient Clinical File: Must have INT_DOD column.\n"
            "Patient Clinical File: Must have DEAD column.\n"
            "Patient Clinical File: Must have SEX column.\n"
        )

        expected_warnings = (
            "Patient Clinical File: Doesn't have PRIMARY_RACE column. "
            "This column will be added\n"
            "Patient Clinical File: Doesn't have SECONDARY_RACE column. "
            "This column will be added\n"
            "Patient Clinical File: Doesn't have TERTIARY_RACE column. "
            "This column will be added\n"
            "Patient Clinical File: Doesn't have ETHNICITY column. "
            "This column will be added\n"
        )
        assert error == expected_errors
        assert warning == expected_warnings


def test_errors__validate(clin_class):
    """
    Test for validation errors
    """
    sampleDf = pd.DataFrame(
        dict(
            SAMPLE_ID=[
                float("nan"),
                "GENIE-SAGE-ID2-1",
                "GENIE-SAGE-ID3-1",
                "GENIE-SAGE-ID4-1",
                "GENIE-SAGE-ID5-1",
            ],
            PATIENT_ID=[
                "GENIE-SAGE-ID6",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                float("nan"),
                "GENIE-SAGE-ID5",
            ],
            AGE_AT_SEQ_REPORT=[10, 100000, ">doo", 100000, 100000],
            ONCOTREE_CODE=["AMPCAD", "TESTIS", "AMPCA", "AMPCA", "UCEC"],
            SAMPLE_TYPE=[1, 2, 3, 4, 6],
            SEQ_ASSAY_ID=[float("nan"), "Sage-1", "SAGE-1", "S-SAGE-1", "SAGE-1"],
            SEQ_DATE=["Jane-2013", "Jan-2013", "Jan-2013", "Jan-2013", "Jan-2013"],
            YEAR_DEATH=["Unknown", "Not Collected", "Not Applicable", 19930, 1990],
            YEAR_CONTACT=["Unknown", "Not Collected", 1990, 1990, 19940],
            INT_CONTACT=[">32485", "<6570", 1990, "Not Collected", ">foobar"],
            INT_DOD=[">32485", "<6570", "Unknown", "Not Collected", "<dense"],
            DEAD=[1, False, "Unknown", "Not Collected", "Not Applicable"],
        )
    )

    patientDf = pd.DataFrame(
        dict(
            PATIENT_ID=[
                "GENIE-SAGE-ID6",
                "GENIE-SAGE-ID2",
                "GENIE-SAGE-ID3",
                float("nan"),
                "GENIE-SAGE-ID5",
            ],
            SEX=[1, 2, 1, 5, float("nan")],
            PRIMARY_RACE=[1, 2, 3, 6, float("nan")],
            SECONDARY_RACE=[1, 2, 3, 6, float("nan")],
            TERTIARY_RACE=[1, 2, 3, 6, float("nan")],
            ETHNICITY=[1, 2, 3, 6, float("nan")],
            BIRTH_YEAR=[1990, 1990, ">90", 1990, 1990],
            CENTER=["FOO", "FOO", "FOO", "FOO", "FOO"],
        )
    )
    clinicalDf = patientDf.merge(sampleDf, on="PATIENT_ID")
    with mock.patch(
        "genie.process_functions.get_oncotree_code_mappings", return_value=onco_map_dict
    ) as mock_get_onco_map:
        error, warning = clin_class._validate(clinicalDf)
        mock_get_onco_map.called_once_with(json_oncotreeurl)

        expectedErrors = (
            "Sample Clinical File: SAMPLE_ID must start with GENIE-SAGE\n"
            "Patient Clinical File: PATIENT_ID must start with GENIE-SAGE\n"
            "Sample Clinical File: PATIENT_ID's much be contained in the "
            "SAMPLE_ID's (ex. SAGE-1 <-> SAGE-1-2)\n"
            "Patient Clinical File: All samples must have associated patient "
            "information and no null patient ids allowed. "
            "These samples are missing patient data: GENIE-SAGE-ID4-1\n"
            "Sample Clinical File: Please double check your "
            "AGE_AT_SEQ_REPORT. It must be an integer, 'Unknown', "
            "'>32485', '<6570'.\n"
            "Sample Clinical File: Please double check that all your "
            "ONCOTREE CODES exist in the mapping. You have 1 samples that "
            "don't map. These are the codes that don't map: AMPCAD\n"
            "Sample Clinical File: Please double check your SAMPLE_TYPE "
            "column.  This column must only be these values: 1, 2, 3, 4, 99\n"
            "Sample Clinical File: Please double check your SEQ_ASSAY_ID "
            "columns, there are empty rows.\n"
            "Sample Clinical File: Please make sure your SEQ_ASSAY_IDs start "
            "with your center abbreviation: S-SAGE-1.\n"
            "Sample Clinical File: SEQ_DATE must be one of five values- "
            "For Jan-March: use Jan-YEAR. "
            "For Apr-June: use Apr-YEAR. "
            "For July-Sep: use Jul-YEAR. "
            "For Oct-Dec: use Oct-YEAR. (ie. Apr-2017) "
            "For values that don't have SEQ_DATES that you want "
            "released use 'release'.\n"
            "Patient Clinical File: Please double check your BIRTH_YEAR "
            "column, it must be an integer in YYYY format <= {year} or "
            "'Unknown', '>89', '<18'.\n"
            "Patient Clinical File: Please double check your YEAR_DEATH "
            "column, it must be an integer in YYYY format <= {year} or "
            "'Unknown', 'Not Collected', 'Not Applicable', "
            "'Not Released', '>89', '<18'.\n"
            "Patient Clinical File: Please double check your YEAR_CONTACT "
            "column, it must be an integer in YYYY format <= {year} or "
            "'Unknown', 'Not Collected', 'Not Released', '>89', '<18'.\n"
            "Patient Clinical File: Please double check your INT_CONTACT "
            "column, it must be an integer, '>32485', '<6570', 'Unknown', "
            "'Not Released' or 'Not Collected'.\n"
            "Patient Clinical File: Please double check your INT_DOD column, "
            "it must be an integer, '>32485', '<6570', 'Unknown', "
            "'Not Collected', 'Not Released' or 'Not Applicable'.\n"
            "Patient Clinical File: Please double check your DEAD column, "
            "it must be True, False, 'Unknown', "
            "'Not Released' or 'Not Collected'.\n"
            "Patient: you have inconsistent redaction and text values in "
            "YEAR_CONTACT, INT_CONTACT.\n"
            "Patient: you have inconsistent redaction and text values in "
            "YEAR_DEATH, INT_DOD.\n"
            "Patient Clinical File: DEAD value is inconsistent with "
            "INT_DOD for at least one patient.\n"
            "Patient Clinical File: Please double check your PRIMARY_RACE "
            "column.  This column must only be these values: 1, 2, 3, 4, 99\n"
            "Patient Clinical File: Please double check your SECONDARY_RACE "
            "column.  This column must only be these values: 1, 2, 3, 4, 99\n"
            "Patient Clinical File: Please double check your TERTIARY_RACE "
            "column.  This column must only be these values: 1, 2, 3, 4, 99\n"
            "Patient Clinical File: Please double check your SEX column.  "
            "This column must only be these values: 1, 2, 99\n"
            "Patient Clinical File: Please double check your ETHNICITY "
            "column.  This column must only be these values: 1, 2, 3, 4, 99\n"
        )
        expectedWarnings = (
            "Sample Clinical File: All patients must have associated sample "
            "information. These patients are missing sample data: GENIE-SAGE-ID6\n"
            "Sample Clinical File: Some SAMPLE_IDs have conflicting SEX and "
            "ONCOTREE_CODES: GENIE-SAGE-ID2-1,GENIE-SAGE-ID5-1\n"
        )
        assert error == expectedErrors.format(year=datetime.datetime.utcnow().year)
        assert warning == expectedWarnings


def test_duplicated__validate(clin_class):
    """
    Test for duplicated SAMPLE_ID error and and in the case that
    both sample and patient
    are uploaded, it could be a duplicated PATIENT_ID error
    """
    patientDf = pd.DataFrame(
        dict(
            PATIENT_ID=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                float("nan"),
            ],
            SEX=[1, 2, 1, 2, float("nan")],
            PRIMARY_RACE=[1, 2, 3, 4, float("nan")],
            SECONDARY_RACE=[1, 2, 3, 4, float("nan")],
            TERTIARY_RACE=[1, 2, 3, 4, float("nan")],
            ETHNICITY=[1, 2, 3, 4, float("nan")],
            BIRTH_YEAR=["Unknown", 1990, 1990, 1990, float("nan")],
            CENTER=["FOO", "FOO", "FOO", "FOO", float("nan")],
            YEAR_CONTACT=["Unknown", "Not Collected", ">89", "<18", float("nan")],
            INT_CONTACT=["Unknown", "Not Collected", ">32485", "<6570", float("nan")],
            YEAR_DEATH=[
                "Unknown",
                "Not Collected",
                "Unknown",
                "Not Applicable",
                float("nan"),
            ],
            INT_DOD=[
                "Unknown",
                "Not Collected",
                "Unknown",
                "Not Applicable",
                float("nan"),
            ],
            DEAD=["Unknown", "Not Collected", "Unknown", False, float("nan")],
        )
    )

    sampleDf = pd.DataFrame(
        dict(
            SAMPLE_ID=[
                "GENIE-SAGE-ID1-1",
                "GENIE-SAGE-ID3-1",
                "GENIE-SAGE-ID4-1",
                float("nan"),
            ],
            PATIENT_ID=[
                "GENIE-SAGE-ID1",
                "GENIE-SAGE-ID3",
                "GENIE-SAGE-ID4",
                float("nan"),
            ],
            AGE_AT_SEQ_REPORT=[100000, 100000, 100000, float("nan")],
            ONCOTREE_CODE=["AMPCA", "UNKNOWN", "AMPCA", float("nan")],
            SAMPLE_TYPE=[1, 3, 4, float("nan")],
            SEQ_ASSAY_ID=["SAGE-1-1", "SAGE-1", "SAGE-1", float("nan")],
            SEQ_DATE=["Jan-2013", "Jul-2013", "Oct-2013", float("nan")],
        )
    )

    clinicalDf = patientDf.merge(sampleDf, on="PATIENT_ID")
    with mock.patch(
        "genie.process_functions.get_oncotree_code_mappings", return_value=onco_map_dict
    ) as mock_get_onco_map:
        error, warning = clin_class._validate(clinicalDf)
        mock_get_onco_map.called_once_with(json_oncotreeurl)
        expectedErrors = (
            "Clinical file(s): No empty rows allowed.\n"
            "Sample Clinical File: No duplicated SAMPLE_ID allowed.\n"
            "If there are no duplicated SAMPLE_IDs, and both sample and "
            "patient files are uploaded, then please check to make sure no "
            "duplicated PATIENT_IDs exist in the patient clinical file.\n"
        )

        assert error == expectedErrors
        assert warning == ""


class fake_oncotree:
    text = json.dumps(
        {
            "TISSUE": {
                "children": {
                    "AMPCA": {
                        "level": 1,
                        "mainType": "Ampullary Cancer",
                        "name": "Ampullary Carcinoma",
                        "children": {
                            "TESTIS": {
                                "level": 2,
                                "mainType": "Testicular Cancer, NOS",
                                "name": "Testis",
                                "children": [],
                            },
                            "UCEC": {
                                "level": 2,
                                "mainType": "Endometrial Cancer",
                                "name": "Endometrial Carcinoma",
                                "children": [],
                            },
                        },
                    }
                }
            }
        }
    )


expected_onco_mapping = {
    "AMPCA": {
        "CANCER_TYPE": "Ampullary Cancer",
        "CANCER_TYPE_DETAILED": "Ampullary Carcinoma",
        "ONCOTREE_PRIMARY_NODE": "AMPCA",
        "ONCOTREE_SECONDARY_NODE": "",
    },
    "TESTIS": {
        "CANCER_TYPE": "Testicular Cancer, NOS",
        "CANCER_TYPE_DETAILED": "Testis",
        "ONCOTREE_PRIMARY_NODE": "AMPCA",
        "ONCOTREE_SECONDARY_NODE": "TESTIS",
    },
    "UCEC": {
        "CANCER_TYPE": "Endometrial Cancer",
        "CANCER_TYPE_DETAILED": "Endometrial Carcinoma",
        "ONCOTREE_PRIMARY_NODE": "AMPCA",
        "ONCOTREE_SECONDARY_NODE": "UCEC",
    },
}


def test_get_oncotree_code_mappings():
    from genie import process_functions

    with mock.patch(
        "genie.process_functions.retry_get_url", return_value=fake_oncotree
    ) as retry_get_url:
        onco_mapping = process_functions.get_oncotree_code_mappings(json_oncotreeurl)
        retry_get_url.called_once_with(json_oncotreeurl)
        assert onco_mapping == expected_onco_mapping


def test__check_year_no_errors():
    """Tests perfect checking year validation function"""
    perfectdf = pd.DataFrame({"BIRTH_YEAR": ["Unknown", 1990, 1990, 1990, 1990]})
    error = genie_registry.clinical._check_year(
        clinicaldf=perfectdf,
        year_col="BIRTH_YEAR",
        filename="Filename",
        allowed_string_values=["Unknown"],
    )
    assert error == ""


def test__check_year_too_big_year():
    """Tests year can't be greater than current year"""
    year_now = datetime.datetime.utcnow().year

    errordf = pd.DataFrame({"BIRTH_YEAR": ["Unknown", year_now + 1, 1990, 1990, 1990]})
    error = genie_registry.clinical._check_year(
        errordf, "BIRTH_YEAR", "Filename", allowed_string_values=["Unknown"]
    )
    assert error == (
        "Filename: Please double check your BIRTH_YEAR column, "
        f"it must be an integer in YYYY format <= {year_now} or 'Unknown'.\n"
    )


def test__check_year_invalid():
    """Tests year can't be greater than current year"""
    year_now = datetime.datetime.utcnow().year

    errordf = pd.DataFrame({"BIRTH_YEAR": ["Unknown", 1990, 1990, 1990, 1990]})
    error = genie_registry.clinical._check_year(errordf, "BIRTH_YEAR", "Filename")
    assert error == (
        "Filename: Please double check your BIRTH_YEAR column, "
        f"it must be an integer in YYYY format <= {year_now}.\n"
    )


def test_remap_clinical_values_sampletype():
    """Test adding of sample type detailed when remapping clinical values"""
    testdf = pd.DataFrame({"SAMPLE_TYPE": [1, 2, 99]})
    expecteddf = pd.DataFrame(
        {
            "SAMPLE_TYPE": ["Test", "Why", "Unknown"],
            "SAMPLE_TYPE_DETAILED": ["non", "asdf", "asdfasdf"],
        }
    )
    remappeddf = genie_registry.clinical.remap_clinical_values(
        testdf, sexdf, no_nan, sexdf, no_nan
    )
    assert expecteddf.equals(remappeddf)


@pytest.mark.parametrize(
    "col", ["SEX", "PRIMARY_RACE", "SECONDARY_RACE", "TERTIARY_RACE", "ETHNICITY"]
)
def test_remap_clinical_values(col):
    """Test Remapping clinical values"""
    testdf = pd.DataFrame({col: [1, 2, 99]})
    expecteddf = pd.DataFrame({col: ["Male", "Female", "Unknown"]})
    remappeddf = genie_registry.clinical.remap_clinical_values(
        testdf, sexdf, sexdf, sexdf, sexdf
    )
    assert expecteddf.equals(remappeddf)


def test__check_int_year_consistency_valid():
    """Test valid vital status consistency"""
    testdf = pd.DataFrame(
        {
            "INT_2": [1, 2, "Unknown", "Unknown"],
            "YEAR_1": [1, 4, "Unknown", 1],
            "FOO_3": [1, 3, "Unknown", 1],
        }
    )
    error = genie_registry.clinical._check_int_year_consistency(
        clinicaldf=testdf, cols=["INT_2", "YEAR_1"], string_vals=["Unknown"]
    )
    assert error == ""


@pytest.mark.parametrize(
    ("inconsistent_df", "expected_err"),
    [
        (
            pd.DataFrame({"INT_2": [1, ">32485", 4], "YEAR_1": [1, 4, 4]}),
            "Patient: you have inconsistent redaction values in INT_2, YEAR_1.\n",
        ),
        (
            pd.DataFrame(
                {"INT_2": [1, 3, "Unknown"], "YEAR_1": [1, 4, "Not Applicable"]}
            ),
            "Patient: you have inconsistent text values in INT_2, YEAR_1.\n",
        ),
        (
            pd.DataFrame(
                {
                    "INT_2": [1, "Unknown", "Unknown"],
                    "YEAR_1": [1, "Not Applicable", "Unknown"],
                }
            ),
            "Patient: you have inconsistent text values in INT_2, YEAR_1.\n",
        ),
        (
            pd.DataFrame({"INT_2": [1, 2, ">32485"], "YEAR_1": [1, 4, "<18"]}),
            "Patient: you have inconsistent redaction values in INT_2, YEAR_1.\n",
        ),
        (
            pd.DataFrame({"INT_2": [1, 2, "<6570"], "YEAR_1": [1, 4, ">89"]}),
            "Patient: you have inconsistent redaction values in INT_2, YEAR_1.\n",
        ),
        (
            pd.DataFrame(
                {
                    "INT_2": ["<6570", "Unknown", "Unknown"],
                    "YEAR_1": [1, "Not Applicable", "Unknown"],
                }
            ),
            "Patient: you have inconsistent redaction and text values in "
            "INT_2, YEAR_1.\n",
        ),
        (
            pd.DataFrame(
                {
                    "INT_2": ["12345", "Unknown", "Unknown"],
                    "YEAR_1": ["Unknown", "Unknown", "Unknown"],
                }
            ),
            "Patient: you have inconsistent text values in INT_2, YEAR_1.\n",
        ),
    ],
)
def test__check_int_year_consistency_inconsistent(inconsistent_df, expected_err):
    """Test inconsistent vital status values"""
    error = genie_registry.clinical._check_int_year_consistency(
        clinicaldf=inconsistent_df,
        cols=["INT_2", "YEAR_1"],
        string_vals=["Unknown", "Not Applicable"],
    )
    assert error == expected_err


@pytest.mark.parametrize(
    "valid_df",
    [
        pd.DataFrame({"INT_DOD": [11111, "Not Applicable"], "DEAD": [True, False]}),
        pd.DataFrame(
            {"INT_DOD": ["Not Applicable", "Not Applicable"], "DEAD": [False, False]}
        ),
    ],
)
def test__check_int_dead_consistency_valid(valid_df):
    """Test valid vital status consistency"""
    error = genie_registry.clinical._check_int_dead_consistency(clinicaldf=valid_df)
    assert error == ""


@pytest.mark.parametrize(
    "inconsistent_df",
    [
        pd.DataFrame(
            {"INT_DOD": ["Not Applicable", "Not Applicable"], "DEAD": [True, False]}
        ),
        pd.DataFrame({"INT_DOD": [1111, "Not Released"], "DEAD": [True, False]}),
        pd.DataFrame({"INT_DOD": [1111, 11111], "DEAD": [True, False]}),
        pd.DataFrame(
            {"INT_DOD": [1111, "Not Released"], "DEAD": [True, "Not Applicable"]}
        ),
    ],
)
def test__check_int_dead_consistency_inconsistent(inconsistent_df):
    """Test valid vital status consistency"""
    error = genie_registry.clinical._check_int_dead_consistency(
        clinicaldf=inconsistent_df
    )
    assert error == (
        "Patient Clinical File: DEAD value is inconsistent with "
        "INT_DOD for at least one patient.\n"
    )



from unittest import mock

import pytest
import synapseclient

from genie_registry.workflow import workflow

syn = mock.create_autospec(synapseclient.Synapse)
workflow_class = workflow(syn, "SAGE")


def test_processing():
    pass


@pytest.fixture(params=[(["foo"]), (["SAGE-test.txt"])])
def filename_fileformat_map(request):
    return request.param


def test_incorrect_validatefilename(filename_fileformat_map):
    filepath_list = filename_fileformat_map
    with pytest.raises(AssertionError):
        workflow_class.validateFilename(filepath_list)


def test_correct_validatefilename():
    assert workflow_class.validateFilename(["SAGE-test.md"]) == "md"



from io import StringIO
import os
import logging

import pandas as pd

from genie.example_filetype_format import FileTypeFormat
from genie import process_functions

logger = logging.getLogger(__name__)


def _check_tsa1_tsa2(df):
    """If maf file has both TSA1 and TSA2,
    TSA1 must equal REF, or TSA1 must equal TSA2.
    """
    tsa2_col_exist = process_functions.checkColExist(df, "TUMOR_SEQ_ALLELE2")
    tsa1_col_exist = process_functions.checkColExist(df, "TUMOR_SEQ_ALLELE1")
    ref_col_exist = process_functions.checkColExist(df, "REFERENCE_ALLELE")
    error = ""
    if tsa2_col_exist and tsa1_col_exist and ref_col_exist:
        tsa1_eq_ref = all(df["TUMOR_SEQ_ALLELE1"] == df["REFERENCE_ALLELE"])
        tsa1_eq_tsa2 = all(df["TUMOR_SEQ_ALLELE1"] == df["TUMOR_SEQ_ALLELE2"])
        if not (tsa1_eq_ref or tsa1_eq_tsa2):
            error = (
                "maf: Contains both "
                "TUMOR_SEQ_ALLELE1 and TUMOR_SEQ_ALLELE2 columns. "
                "All values in TUMOR_SEQ_ALLELE1 must match all values in "
                "REFERENCE_ALLELE or all values in TUMOR_SEQ_ALLELE2.\n"
            )
    return error


def _check_allele_col(df, col):
    """
    Check the Allele column is correctly formatted.

    Args:
        df: mutation dataframe
        col: Column header name

    Returns:
        error, warning

    """
    col_exist = process_functions.checkColExist(df, col)
    error = ""
    warning = ""
    if col_exist:
        # CHECK: The value "NA" can't be used as a placeholder
        if sum(df[col].fillna("") == "NA") > 0:
            warning = (
                "maf: "
                f"{col} column contains 'NA' values, "
                "which cannot be placeholders for blank values.  "
                "Please put in empty strings for blank values.\n"
            )
        # CHECK: There can't be any null values
        if sum(df[col].isnull()) > 0:
            error = f"maf: {col} can't have any blank or null values.\n"

    return error, warning


class maf(FileTypeFormat):
    """
    MAF file format validation / processing
    """

    _fileType = "maf"

    _process_kwargs = []

    def _validateFilename(self, filePath):
        """
        Validates filename.  Should be
        data_mutations_extended_CENTER.txt
        """
        assert os.path.basename(filePath[0]) == "data_mutations_extended_{}.txt".format(
            self.center
        )

    def process_steps(self, df):
        """The processing of maf files is specific to GENIE, so
        not included in this function"""
        logger.info(
            "Please run with `--process mutation` parameter "
            "if you want to reannotate the mutation files"
        )
        return None

    def _validate(self, mutationDF):
        """
        This function validates the mutation file to make sure it
        adheres to the mutation SOP.

        Args:
            mutationDF: mutation dataframe

        Returns:
            Text with all the errors in the mutation file
        """

        first_header = ["CHROMOSOME", "HUGO_SYMBOL", "TUMOR_SAMPLE_BARCODE"]
        SP = self._fileType == "mafSP"
        if SP:
            correct_column_headers = [
                "CHROMOSOME",
                "START_POSITION",
                "REFERENCE_ALLELE",
                "TUMOR_SAMPLE_BARCODE",
                "TUMOR_SEQ_ALLELE2",
            ]
            # T_REF_COUNT + T_ALT_COUNT = T_DEPTH
        else:
            correct_column_headers = [
                "CHROMOSOME",
                "START_POSITION",
                "REFERENCE_ALLELE",
                "TUMOR_SAMPLE_BARCODE",
                "T_ALT_COUNT",
                "TUMOR_SEQ_ALLELE2",
            ]
            # T_REF_COUNT + T_ALT_COUNT = T_DEPTH
        optional_headers = ["T_REF_COUNT", "N_DEPTH", "N_REF_COUNT", "N_ALT_COUNT"]

        mutationDF.columns = [col.upper() for col in mutationDF.columns]

        # total_error = ""
        total_error = StringIO()
        warning = StringIO()

        # CHECK: Everything in correct_column_headers must be in mutation file
        if not all(
            [
                process_functions.checkColExist(mutationDF, i)
                for i in correct_column_headers
            ]
        ):
            total_error.write(
                "maf: Must at least have these headers: {}. "
                "If you are writing your maf file with R, please make"
                "sure to specify the 'quote=FALSE' parameter.\n".format(
                    ",".join(
                        [
                            i
                            for i in correct_column_headers
                            if i not in mutationDF.columns.values
                        ]
                    )
                )
            )
        else:
            # CHECK: First column must be in the first_header list
            if mutationDF.columns[0] not in first_header:
                total_error.write(
                    "maf: First column header must be "
                    "one of these: {}.\n".format(", ".join(first_header))
                )
            # No duplicated values
            primary_cols = [
                "CHROMOSOME",
                "START_POSITION",
                "REFERENCE_ALLELE",
                "TUMOR_SAMPLE_BARCODE",
                "TUMOR_SEQ_ALLELE2",
            ]
            # Strip white space if string column
            for col in primary_cols:
                if mutationDF[col].dtype == object:
                    mutationDF[col] = mutationDF[col].str.strip()
            duplicated_idx = mutationDF.duplicated(primary_cols)
            # Find samples with duplicated variants
            duplicated_variants = (
                mutationDF["TUMOR_SAMPLE_BARCODE"][duplicated_idx]
                .unique()
                .astype(str)
                .tolist()
            )

            if duplicated_idx.any():
                total_error.write(
                    "maf: Must not have duplicated variants. "
                    "Samples with duplicated variants: "
                    f"{', '.join(duplicated_variants)}\n"
                )

        t_depth_exists = process_functions.checkColExist(mutationDF, "T_DEPTH")
        t_ref_exists = process_functions.checkColExist(mutationDF, "T_REF_COUNT")
        if not t_depth_exists and not t_ref_exists and not SP:
            total_error.write("maf: If missing T_DEPTH, must have T_REF_COUNT!\n")
        numerical_cols = [
            "T_DEPTH",
            "T_ALT_COUNT",
            "T_REF_COUNT",
            "N_DEPTH",
            "N_REF_COUNT",
            "N_ALT_COUNT",
        ]
        for col in numerical_cols:
            col_exists = process_functions.checkColExist(mutationDF, col)
            if col_exists:
                # Since NA is an allowed value, when reading in the dataframe
                # the 'NA' string is not converted.  This will convert all
                # 'NA' values in the numerical columns into actual float('nan')
                mutationDF.loc[mutationDF[col] == "NA", col] = float("nan")
                # Attempt to convert column to float
                try:
                    mutationDF[col] = mutationDF[col].astype(float)
                except ValueError:
                    pass
                if mutationDF[col].dtype not in [int, float]:
                    total_error.write(f"maf: {col} must be a numerical column.\n")

        # CHECK: Must have TUMOR_SEQ_ALLELE2
        error, warn = _check_allele_col(mutationDF, "TUMOR_SEQ_ALLELE2")
        total_error.write(error)
        warning.write(warn)

        # CHECK: Mutation file would benefit from columns in optional_headers
        if (
            not all(
                [
                    process_functions.checkColExist(mutationDF, i)
                    for i in optional_headers
                ]
            )
            and not SP
        ):
            warning.write(
                "maf: Does not have the column headers that can give extra "
                "information to the processed maf: {}.\n".format(
                    ", ".join(
                        [
                            i
                            for i in optional_headers
                            if i not in mutationDF.columns.values
                        ]
                    )
                )
            )

        # CHECK: Must have REFERENCE_ALLELE
        error, warn = _check_allele_col(mutationDF, "REFERENCE_ALLELE")
        total_error.write(error)
        warning.write(warn)

        if process_functions.checkColExist(mutationDF, "CHROMOSOME"):
            # CHECK: Chromosome column can't have any values that start
            # with chr or have any WT values
            invalid_values = [
                str(i).startswith("chr") or str(i) == "WT"
                for i in mutationDF["CHROMOSOME"]
            ]
            if sum(invalid_values) > 0:
                total_error.write(
                    "maf: CHROMOSOME column cannot have any values that "
                    "start with 'chr' or any 'WT' values.\n"
                )

        error = _check_tsa1_tsa2(mutationDF)
        total_error.write(error)

        if process_functions.checkColExist(mutationDF, "TUMOR_SAMPLE_BARCODE"):
            error = process_functions.validate_genie_identifier(
                identifiers=mutationDF["TUMOR_SAMPLE_BARCODE"],
                center=self.center,
                filename="maf",
                col="TUMOR_SAMPLE_BARCODE",
            )
            total_error.write(error)

        return total_error.getvalue(), warning.getvalue()

    def _get_dataframe(self, filePathList):
        """Get mutation dataframe"""
        # Must do this because pandas.read_csv will allow for a file to
        # have more column headers than content.  E.g.
        # A,B,C,D,E
        # 1,2
        # 2,3
        with open(filePathList[0], "r") as maf_f:
            firstline = maf_f.readline()
            if firstline.startswith("#"):
                firstline = maf_f.readline()
            secondline = maf_f.readline()

        if len(firstline.split("\t")) != len(secondline.split("\t")):
            raise ValueError(
                "Number of fields in a line do not match the "
                "expected number of columns"
            )

        mutationdf = pd.read_csv(
            filePathList[0],
            sep="\t",
            comment="#",
            # Keep the value 'NA'
            na_values=[
                "-1.#IND",
                "1.#QNAN",
                "1.#IND",
                "-1.#QNAN",
                "#N/A N/A",
                "NaN",
                "#N/A",
                "N/A",
                "#NA",
                "NULL",
                "-NaN",
                "nan",
                "-nan",
                "",
            ],
            keep_default_na=False,
            # This is to check if people write files
            # with R, quote=T
            quoting=3,
            # Retain completely blank lines so that
            # validator will cause the file to fail
            skip_blank_lines=False,
        )
        return mutationdf



"""Process mutation files"""
import logging
import os
import shutil
import subprocess
import tempfile

import pandas as pd
import synapseclient  # lgtm [py/import-and-import-from]
from synapseclient import Synapse
from synapseclient.core.exceptions import SynapseTimeoutError

from . import process_functions

logger = logging.getLogger(__name__)

# Some columns are already capitalized, so they aren't included here
MAF_COL_MAPPING = {
    "HUGO_SYMBOL": "Hugo_Symbol",
    "ENTREZ_GENE_ID": "Entrez_Gene_Id",
    "CENTER": "Center",
    "NCBI_BUILD": "NCBI_Build",
    "CHROMOSOME": "Chromosome",
    "START_POSITION": "Start_Position",
    "END_POSITION": "End_Position",
    "STRAND": "Strand",
    "VARIANT_CLASSIFICATION": "Variant_Classification",
    "VARIANT_TYPE": "Variant_Type",
    "REFERENCE_ALLELE": "Reference_Allele",
    "TUMOR_SEQ_ALLELE1": "Tumor_Seq_Allele1",
    "TUMOR_SEQ_ALLELE2": "Tumor_Seq_Allele2",
    "DBSNP_RS": "dbSNP_RS",
    "DBSNP_VAL_STATUS": "dbSNP_Val_Status",
    "TUMOR_SAMPLE_BARCODE": "Tumor_Sample_Barcode",
    "MATCHED_NORM_SAMPLE_BARCODE": "Matched_Norm_Sample_Barcode",
    "MATCH_NORM_SEQ_ALLELE1": "Match_Norm_Seq_Allele1",
    "MATCH_NORM_SEQ_ALLELE2": "Match_Norm_Seq_Allele2",
    "TUMOR_VALIDATION_ALLELE1": "Tumor_Validation_Allele1",
    "TUMOR_VALIDATION_ALLELE2": "Tumor_Validation_Allele2",
    "MATCH_NORM_VALIDATION_ALLELE1": "Match_Norm_Validation_Allele1",
    "MATCH_NORM_VALIDATION_ALLELE2": "Match_Norm_Validation_Allele2",
    "VERIFICATION_STATUS": "Verification_Status",
    "VALIDATION_STATUS": "Validation_Status",
    "MUTATION_STATUS": "Mutation_Status",
    "SEQUENCING_PHASE": "Sequencing_Phase",
    "SEQUENCE_SOURCE": "Sequence_Source",
    "VALIDATION_METHOD": "Validation_Method",
    "SCORE": "Score",
    "BAM_FILE": "BAM_File",
    "SEQUENCER": "Sequencer",
    "T_REF_COUNT": "t_ref_count",
    "T_ALT_COUNT": "t_alt_count",
    "N_REF_COUNT": "n_ref_count",
    "N_ALT_COUNT": "n_alt_count",
    "ALLELE": "Allele",
    "AMINO_ACID_CHANGE": "amino_acid_change",
    "AMINO_ACIDS": "Amino_acids",
    "CDS_POSITION": "CDS_position",
    "CODONS": "Codons",
    "CONSEQUENCE": "Consequence",
    "EXISTING_VARIATION": "Existing_variation",
    "EXON_NUMBER": "Exon_Number",
    "FEATURE": "Feature",
    "FEATURE_TYPE": "Feature_type",
    "GENE": "Gene",
    "HGVSC": "HGVSc",
    "HGVSP": "HGVSp",
    "HGVSP_SHORT": "HGVSp_Short",
    "HOTSPOT": "Hotspot",
    "MA:FIMPACT": "MA:FImpact",
    "MA:LINK.MSA": "MA:link.MSA",
    "MA:LINK.PDB": "MA:link.PDB",
    "MA:LINK.VAR": "MA:link.var",
    "MA:PROTEIN.CHANGE": "MA:protein.change",
    "POLYPHEN": "PolyPhen",
    "PROTEIN_POSITION": "Protein_position",
    "REFSEQ": "RefSeq",
    "TRANSCRIPT": "transcript",
    "TRANSCRIPT_ID": "Transcript_ID",
    "ALL_EFFECTS": "all_effects",
    "CDNA_CHANGE": "cdna_change",
    "CDNA_POSITION": "cDNA_position",
    "N_DEPTH": "n_depth",
    "T_DEPTH": "t_depth",
}

KNOWN_STRING_COLS = [
    "IS_NEW",
    "ALLELE_NUM",
    "Chromosome",
    "CLIN_SIG",
    "MOTIF_NAME",
    "HIGH_INF_POS",
    "MINIMISED",
    "CHROMOSOME",
    "VERIFICATION_STATUS",
    "VALIDATION_STATUS",
    "MUTATION_STATUS",
    "SEQUENCE_SOURCE",
    "SEQUENCER",
    "REPORT_AF",
    "CDNA_CHANGE",
    "AMINO_ACID_CHANGE",
    "TRANSCRIPT",
    "STRAND_VEP",
    "HGNC_ID",
    "PUBMED",
    "PICK",
]


def _rename_column_headers(dataframe: pd.DataFrame, col_map: dict) -> pd.DataFrame:
    """Rename dataframe column headers

    Args:
        dataframe: pandas dataframe
        col_map: Column Mapping {column_name: new_column_name,}

    Returns:
        Dataframe with new columns

    """
    dataframe = dataframe.rename(columns=col_map)
    return dataframe


def _convert_to_str_dtype(column_types, known_string_cols):
    """Sometimes the deteremined dtype is incorrect based off the first
    100 rows, update the incorrect dtypes.
    """
    for str_col in known_string_cols:
        if column_types.get(str_col):
            column_types[str_col] = "object"
    return column_types


def determine_dtype(path: str):
    """Reads in a dataframe partially and determines the dtype of columns"""
    subset_df = pd.read_csv(path, nrows=100, sep="\t")
    dtypes = subset_df.dtypes
    colnames = dtypes.index
    types = [i.name for i in dtypes.values]
    column_types = dict(zip(colnames, types))
    return column_types


def move_maf(mutation_path, input_files_dir):
    """Moves maf files into processing directory. Maf file's column headers
    are renamed if necessary"""
    header_df = pd.read_csv(mutation_path, sep="\t", index_col=0, nrows=1, comment="#")
    # If any column headers need to be remapped, remap
    if sum(header_df.columns.isin(MAF_COL_MAPPING.keys())) > 0:
        filename = os.path.basename(mutation_path)
        column_types = determine_dtype(mutation_path)
        new_column_types = _convert_to_str_dtype(column_types, KNOWN_STRING_COLS)
        mafdf = pd.read_csv(mutation_path, sep="\t", dtype=new_column_types)
        mafdf = _rename_column_headers(mafdf, col_map=MAF_COL_MAPPING)
        mafdf.to_csv(os.path.join(input_files_dir, filename), sep="\t", index=False)
    else:
        shutil.copy(mutation_path, input_files_dir)


def move_mutation(mutation_path, input_files_dir):
    """Move mutation file into processing directory"""
    # If mutation file is vcf, just copy
    if mutation_path.endswith(".vcf"):
        shutil.copy(mutation_path, input_files_dir)
    else:
        move_maf(mutation_path, input_files_dir)


def process_mutation_workflow(
    syn: Synapse,
    center: str,
    validfiles: pd.DataFrame,
    genie_config: dict,
    workdir: str,
) -> str:
    """Process vcf/maf workflow

    Args:
        syn: Synapse connection
        center: Center name
        validfiles: Center validated files
        genie_config: GENIE configuration.
        workdir: Working directory

    Returns:
        Annotated Maf Path

    """
    # Get valid files
    mutation_files = validfiles["fileType"].isin(["maf", "vcf"])
    valid_mutation_files = validfiles["path"][mutation_files].tolist()
    # If there are no valid mutation files, return
    if not valid_mutation_files:
        logger.info("No mutation data")
        return
    # Certificate to use GENIE Genome Nexus
    syn.get(
        "syn22053204",
        ifcollision="overwrite.local",
        downloadLocation=genie_config["genie_annotation_pkg"],
    )
    # Genome Nexus Jar file
    syn.get(
        "syn22084320",
        ifcollision="overwrite.local",
        downloadLocation=genie_config["genie_annotation_pkg"],
    )

    annotated_maf_path = annotate_mutation(
        center=center,
        mutation_files=valid_mutation_files,
        genie_annotation_pkg=genie_config["genie_annotation_pkg"],
        workdir=workdir,
    )

    maf_tableid = genie_config["vcf2maf"]
    flatfiles_synid = genie_config["centerMaf"]
    # Split into narrow maf and store into db / flat file
    split_and_store_maf(
        syn=syn,
        center=center,
        maf_tableid=maf_tableid,
        annotated_maf_path=annotated_maf_path,
        flatfiles_synid=flatfiles_synid,
        workdir=workdir,
    )

    return annotated_maf_path


def annotate_mutation(
    center: str, mutation_files: list, genie_annotation_pkg: str, workdir: str
) -> str:
    """Process vcf/maf files

    Args:
        center: Center name
        mutation_files: list of mutation files
        genie_annotation_pkg: Path to GENIE annotation package

    Returns:
        Path to final maf
    """
    input_files_dir = tempfile.mkdtemp(dir=workdir)
    output_files_dir = tempfile.mkdtemp(dir=workdir)

    for mutation_file in mutation_files:
        move_mutation(mutation_file, input_files_dir)

    merged_maf_path = os.path.join(
        output_files_dir, f"data_mutations_extended_{center}.txt"
    )
    annotater_cmd = [
        "bash",
        os.path.join(genie_annotation_pkg, "annotation_suite_wrapper.sh"),
        f"-i={input_files_dir}",
        f"-o={output_files_dir}",
        f"-m={merged_maf_path}",
        f"-c={center}",
        "-s=WXS",
        f"-p={genie_annotation_pkg}",
    ]

    subprocess.check_call(annotater_cmd)

    return merged_maf_path


def append_or_createdf(dataframe: pd.DataFrame, filepath: str):
    """Creates a file with the dataframe or appends to a existing file.

    Args:
        df: pandas.dataframe to write out
        filepath: Filepath to append or create

    """
    if not os.path.exists(filepath) or os.stat(filepath).st_size == 0:
        dataframe.to_csv(filepath, sep="\t", index=False)
    else:
        dataframe.to_csv(filepath, sep="\t", mode="a", index=False, header=None)


def store_full_maf(syn: Synapse, filepath: str, parentid: str):
    """Stores full maf file

    Args:
        syn: Synapse connection
        filepath: Path to file
        parentid: Synapse container id

    """
    syn.store(synapseclient.File(filepath, parentId=parentid))


def store_narrow_maf(syn: Synapse, filepath: str, maf_tableid: str):
    """
    Stores the narrow maf in Synapse Table

    Args:
        syn: Synapse connection
        filepath: Path to maf file
        maf_tableid: database synid

    """
    logger.info(f"STORING {filepath}")
    # database = syn.get(maf_tableid)
    try:
        update_table = synapseclient.Table(maf_tableid, filepath, separator="\t")
        syn.store(update_table)
    except SynapseTimeoutError:
        # This error occurs because of waiting for table to index.
        # Don't worry about this.
        pass


def format_maf(mafdf: pd.DataFrame, center: str) -> pd.DataFrame:
    """Format maf file, shortens the maf file length

    Args:
        mafdf: mutation dataframe
        center: Center name

    Returns:
        Formatted mutation dataframe"""
    mafdf["Center"] = center
    # Leaving here for safe guarding.
    mafdf["Tumor_Sample_Barcode"] = [
        process_functions.checkGenieId(i, center) for i in mafdf["Tumor_Sample_Barcode"]
    ]
    mafdf["Sequence_Source"] = float("nan")
    mafdf["Sequencer"] = float("nan")
    mafdf["Validation_Status"][
        mafdf["Validation_Status"].isin(["Unknown", "unknown"])
    ] = ""

    return mafdf


def split_and_store_maf(
    syn: "Synapse",
    center: str,
    maf_tableid: str,
    annotated_maf_path: str,
    flatfiles_synid: str,
    workdir: str,
):
    """Separates annotated maf file into narrow and full maf and stores them

    Args:
        syn: Synapse connection
        center: Center
        maf_tableid: Mutation table synapse id
        annotated_maf_path: Annotated maf
        flatfiles_synid: GENIE flat files folder

    """
    narrow_maf_cols = [
        col["name"]
        for col in syn.getTableColumns(maf_tableid)
        if col["name"] != "inBED"
    ]
    full_maf_path = os.path.join(
        workdir, center, "staging", f"data_mutations_extended_{center}.txt"
    )
    narrow_maf_path = os.path.join(
        workdir, center, "staging", f"data_mutations_extended_{center}_MAF_narrow.txt"
    )
    maf_chunks = pd.read_csv(annotated_maf_path, sep="\t", chunksize=100000)

    for maf_chunk in maf_chunks:
        maf_chunk = format_maf(maf_chunk, center)
        append_or_createdf(maf_chunk, full_maf_path)
        narrow_maf_chunk = maf_chunk[narrow_maf_cols]
        append_or_createdf(narrow_maf_chunk, narrow_maf_path)

    store_narrow_maf(syn, narrow_maf_path, maf_tableid)
    # Store MAF flat file into synapse
    store_full_maf(syn, full_maf_path, flatfiles_synid)



"""Initialize GENIE registry"""
# Import logging last to not take in synapseclient logging
import logging

from . import (
    assay,
    bed,
    cbs,
    clinical,
    cna,
    fusions,
    maf,
    mutationsInCis,
    patientRetraction,
    sampleRetraction,
    seg,
    structural_variant,
    vcf,
    workflow,
)

# Set genie registry to be the same as main genie codebase
from genie import __version__

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)
logging.getLogger("keyring").setLevel(logging.WARNING)

__all__ = [
    "__version__",
    "assay",
    "bed",
    "cbs",
    "clinical",
    "cna",
    "fusions",
    "maf",
    "mutationsInCis",
    "patientRetraction",
    "sampleRetraction",
    "seg",
    "structural_variant",
    "vcf",
    "workflow",
]



"""Test assay information validation and processing"""
import copy
from unittest.mock import patch, create_autospec

import pandas as pd
import pytest
import synapseclient

from genie_registry.assay import Assayinfo
from genie import process_functions

GDC_DATA_DICT = {
    "properties": {
        "library_strategy": {"enum": ["value1", "value2"]},
        "library_selection": {"enum": ["value1", "value2"]},
        "platform": {"enum": ["value1", "value2"]},
        "instrument_model": {"enum": ["value1", "value2"]},
        "target_capture_kit": {"enum": ["value1", "value2"]},
    }
}

SYN = create_autospec(synapseclient.Synapse)
ASSAY_INFO = Assayinfo(SYN, "SAGE")


def test_filetype():
    assert ASSAY_INFO._fileType == "assayinfo"


def test_invalidname__validatefilename():
    """Assertion error thrown for wrong filename"""
    with pytest.raises(AssertionError):
        ASSAY_INFO.validateFilename(["foo"])


def test_correct__validatefilename():
    """Filetype returned when filename valid"""
    assert ASSAY_INFO.validateFilename(["assay_information.yaml"]) == "assayinfo"


def test_validinput__validate():
    """Valid input should have no errors or warnings"""
    assay_info_dict = {
        "SEQ_ASSAY_ID": ["SAGE-1", "SAGE-3"],
        "is_paired_end": [True, False],
        "library_strategy": ["value1", "value2"],
        "library_selection": ["value1", "value2"],
        "platform": ["value1", "value2"],
        "instrument_model": ["value1", "value2"],
        "target_capture_kit": ["value1", "value2"],
        "variant_classifications": ["Frame_Shift_Ins", "Frame_Shift_Ins"],
        "read_length": [22, float("nan")],
        "number_of_genes": [5, 20],
        "gene_padding": [10, None],
        "calling_strategy": ["tumor_only", "tumor_normal"],
        "specimen_tumor_cellularity": [">10%", ">20%"],
        "alteration_types": ["snv;small_indels", "intragenic_cna"],
        "preservation_technique": ["FFPE", "FFPE;fresh_frozen"],
        "coverage": ["hotspot_regions;introns", "introns"],
    }
    uniq_seq_df = pd.DataFrame({"seq": ["SAGE-1", "SAGE-3"]})
    assay_info_df = pd.DataFrame(assay_info_dict)
    test_dict = copy.deepcopy(GDC_DATA_DICT)
    with patch.object(
        process_functions, "get_synid_database_mappingdf", return_value="syn123"
    ), patch.object(
        process_functions, "getDatabaseSynId", return_value="syn1234"
    ), patch.object(
        process_functions, "get_syntabledf", return_value=uniq_seq_df
    ), patch.object(
        process_functions, "get_gdc_data_dictionary", return_value=test_dict
    ) as patch_get_gdc:
        error, warning = ASSAY_INFO._validate(assay_info_df, "syn9999")
        assert error == ""
        assert warning == ""
        patch_get_gdc.assert_called()


def test_case__validate():
    """Valid input should have no errors or warnings"""
    assay_info_dict = {
        "SEQ_ASSAY_ID": ["sage-1", "SAGE-3"],
        "is_paired_end": [True, False],
        "library_strategy": ["value1", "value2"],
        "library_selection": ["value1", "value2"],
        "platform": ["value1", "value2"],
        "instrument_model": ["value1", "value2"],
        "target_capture_kit": ["value1", "value2"],
        "variant_classifications": ["Frame_Shift_Ins", "Frame_Shift_Ins"],
        "read_length": [22, float("nan")],
        "number_of_genes": [5, 20],
        "gene_padding": [10, None],
        "calling_strategy": ["tumor_only", "tumor_normal"],
        "specimen_tumor_cellularity": [">10%", ">20%"],
        "alteration_types": ["snv;small_indels", "intragenic_cna"],
        "preservation_technique": ["FFPE", "FFPE;fresh_frozen"],
        "coverage": ["hotspot_regions;introns", "introns"],
    }
    uniq_seq_df = pd.DataFrame({"seq": ["SAGE-1", "SAGE-3"]})
    assay_info_df = pd.DataFrame(assay_info_dict)
    test_dict = copy.deepcopy(GDC_DATA_DICT)
    with patch.object(
        process_functions, "get_synid_database_mappingdf", return_value="syn123"
    ), patch.object(
        process_functions, "getDatabaseSynId", return_value="syn1234"
    ), patch.object(
        process_functions, "get_syntabledf", return_value=uniq_seq_df
    ), patch.object(
        process_functions, "get_gdc_data_dictionary", return_value=test_dict
    ) as patch_get_gdc:
        error, warning = ASSAY_INFO._validate(assay_info_df, "syn9999")
        assert error == ""
        assert warning == ""
        patch_get_gdc.assert_called()


def test_underscore__validate():
    """Valid input should have no errors or warnings"""
    assay_info_dict = {
        "SEQ_ASSAY_ID": ["SAGE_1", "SAGE-3"],
        "is_paired_end": [True, False],
        "library_strategy": ["value1", "value2"],
        "library_selection": ["value1", "value2"],
        "platform": ["value1", "value2"],
        "instrument_model": ["value1", "value2"],
        "target_capture_kit": ["value1", "value2"],
        "variant_classifications": ["Frame_Shift_Ins", "Frame_Shift_Ins"],
        "read_length": [22, float("nan")],
        "number_of_genes": [5, 20],
        "gene_padding": [10, None],
        "calling_strategy": ["tumor_only", "tumor_normal"],
        "specimen_tumor_cellularity": [">10%", ">20%"],
        "alteration_types": ["snv;small_indels", "intragenic_cna"],
        "preservation_technique": ["FFPE", "FFPE;fresh_frozen"],
        "coverage": ["hotspot_regions;introns", "introns"],
    }
    uniq_seq_df = pd.DataFrame({"seq": ["SAGE-1", "SAGE-3"]})
    assay_info_df = pd.DataFrame(assay_info_dict)
    test_dict = copy.deepcopy(GDC_DATA_DICT)
    with patch.object(
        process_functions, "get_synid_database_mappingdf", return_value="syn123"
    ), patch.object(
        process_functions, "getDatabaseSynId", return_value="syn1234"
    ), patch.object(
        process_functions, "get_syntabledf", return_value=uniq_seq_df
    ), patch.object(
        process_functions, "get_gdc_data_dictionary", return_value=test_dict
    ) as patch_get_gdc:
        error, warning = ASSAY_INFO._validate(assay_info_df, "syn9999")
        assert error == ""
        assert warning == ""
        patch_get_gdc.assert_called()


def test__missingcols__validate():
    """Test missing columns"""
    assay_info_df = pd.DataFrame()
    test_dict = copy.deepcopy(GDC_DATA_DICT)
    with patch.object(
        process_functions, "get_gdc_data_dictionary", return_value=test_dict
    ) as patch_get_gdc:
        error, warning = ASSAY_INFO._validate(assay_info_df, "syn99999")
    expected_errors = (
        "Assay_information.yaml: Must have SEQ_ASSAY_ID column.\n"
        "Assay_information.yaml: Must have is_paired_end column.\n"
        "Assay_information.yaml: Must have library_selection column.\n"
        "Assay_information.yaml: Must have library_strategy column.\n"
        "Assay_information.yaml: Must have platform column.\n"
        "Assay_information.yaml: Must have instrument_model column.\n"
        "Assay_information.yaml: Must have target_capture_kit column.\n"
        "Assay_information.yaml: Must have read_length column.\n"
        "Assay_information.yaml: Must have number_of_genes column.\n"
        "Assay_information.yaml: Must have calling_strategy column.\n"
        "Assay_information.yaml: "
        "Must have specimen_tumor_cellularity column.\n"
        "Assay_information.yaml: Must have alteration_types column.\n"
        "Assay_information.yaml: Must have preservation_technique column.\n"
        "Assay_information.yaml: Must have coverage column.\n"
    )
    assert error == expected_errors

    expected_warnings = (
        "Assay_information.yaml: Doesn't have variant_classifications column. "
        "This column will be added\n"
        "Assay_information.yaml: gene_padding is "
        "by default 10 if not specified.\n"
    )
    assert warning == expected_warnings
    patch_get_gdc.assert_called()


def test_fillcols__process():
    """
    Standardization of SEQ_ASSAY_ID
    Add in CENTER, gene_padding, and variant_classifications if missing
    """

    assay_info_dict = {"SEQ_ASSAY_ID": ["SAGE-Foo_1"], "SEQ_PIPELINE_ID": ["SAGE_Foo"]}
    assay_info_df = pd.DataFrame(assay_info_dict)
    processed_assay_df = ASSAY_INFO._process(assay_info_df)
    expected_assay_df = pd.DataFrame(
        {
            "SEQ_ASSAY_ID": ["SAGE-FOO-1"],
            "SEQ_PIPELINE_ID": ["SAGE-FOO"],
            "gene_padding": [10],
            "variant_classifications": [float("nan")],
            "CENTER": ["SAGE"],
        }
    )

    assert expected_assay_df.equals(processed_assay_df[expected_assay_df.columns])


def test_default10__process():
    """
    gene_padding default 10
    """

    assay_info_dict = {
        "SEQ_ASSAY_ID": ["SAGE-1", "SAGE-2"],
        "SEQ_PIPELINE_ID": ["SAGE-1", "SAGE-2"],
        "gene_padding": [20, float("nan")],
        "variant_classifications": ["test", "test"],
    }
    assay_info_df = pd.DataFrame(assay_info_dict)
    processed_assay_df = ASSAY_INFO._process(assay_info_df)
    expected_assay_df = pd.DataFrame(
        {
            "SEQ_ASSAY_ID": ["SAGE-1", "SAGE-2"],
            "SEQ_PIPELINE_ID": ["SAGE-1", "SAGE-2"],
            "gene_padding": [20, 10],
            "variant_classifications": ["test", "test"],
            "CENTER": ["SAGE", "SAGE"],
        }
    )
    assert expected_assay_df.equals(processed_assay_df[expected_assay_df.columns])


def test_invalid__validate():
    assay_info_dict = {
        "SEQ_ASSAY_ID": ["SAGE-1", "SAG-2"],
        "is_paired_end": [True, "foo"],
        "library_strategy": ["foo", "ChIP-Seq"],
        "library_selection": ["foo", "PCR"],
        "platform": ["foo", "Illumina"],
        "instrument_model": ["foo", "Illumina HiSeq 4000"],
        "variant_classifications": ["foo", "Frame_Shift_Ins"],
        "target_capture_kit": ["foo", "doo"],
        "read_length": [22, "foo"],
        "number_of_genes": [5, "foo"],
        "gene_padding": [10, "foo"],
        "calling_strategy": ["tumor_ony", "tumor_normal"],
        "specimen_tumor_cellularity": [">10", ">20%"],
        "alteration_types": ["snv;small_indel", "intragenic_cna"],
        "preservation_technique": ["FPE", "FFPE;fresh_frozen"],
        "coverage": ["hotsot_regions;introns", "introns"],
    }
    assay_info_df = pd.DataFrame(assay_info_dict)
    uniq_seq_df = pd.DataFrame({"seq": ["SAGE-1", "SAGE-2"]})

    # Must use deepcopy because a dict.copy is a shallow copy
    # Which just points to reference keys
    test_dict = copy.deepcopy(GDC_DATA_DICT)
    with patch.object(
        process_functions, "get_synid_database_mappingdf", return_value="syn123"
    ), patch.object(
        process_functions, "getDatabaseSynId", return_value="syn1234"
    ), patch.object(
        process_functions, "get_syntabledf", return_value=uniq_seq_df
    ), patch.object(
        process_functions, "get_gdc_data_dictionary", return_value=test_dict
    ) as patch_get_gdc:
        error, warning = ASSAY_INFO._validate(assay_info_df, "syn9999")
        expected_errors = (
            "Assay_information.yaml: "
            "Please make sure all your SEQ_ASSAY_IDs start with your "
            "center abbreviation.\n"
            "Assay_information.yaml: You are missing SEQ_ASSAY_IDs: SAGE-2\n"
            "Assay_information.yaml: "
            "Please double check your is_paired_end column.  "
            "This column must only be these values: True, False\n"
            "Assay_information.yaml: "
            "Please double check your library_selection column.  "
            "This column must only be these values: value1, value2\n"
            "Assay_information.yaml: "
            "Please double check your library_strategy column.  "
            "This column must only be these values: value1, value2\n"
            "Assay_information.yaml: "
            "Please double check your platform column.  "
            "This column must only be these values: value1, value2\n"
            "Assay_information.yaml: "
            "Please double check your instrument_model column.  "
            "This column must only be these values: value1, value2, "
            "Illumina NovaSeq 6000, None\n"
            "Assay_information.yaml: "
            "Please double check your variant_classifications column.  "
            "This column must only be these values: Splice_Site, "
            "Nonsense_Mutation, Frame_Shift_Del, Frame_Shift_Ins, "
            "Nonstop_Mutation, Translation_Start_Site, In_Frame_Ins, "
            "In_Frame_Del, Missense_Mutation, Intron, Splice_Region, "
            "Silent, RNA, 5'UTR, 3'UTR, IGR, 5'Flank, 3'Flank, None\n"
            "Assay_information.yaml: "
            "Please double check your read_length.  "
            "It must be an integer or null.\n"
            "Assay_information.yaml: "
            "Please double check your number_of_genes. "
            "It must be an integer.\n"
            "Assay_information.yaml: "
            "Please double check your gene_padding. "
            "It must be an integer or blank.\n"
            "Assay_information.yaml: "
            "Please double check your calling_strategy column.  This "
            "column must only be these values: tumor_only, tumor_normal, "
            "plasma_normal\n"
            "Assay_information.yaml: "
            "Please double check your specimen_tumor_cellularity. "
            "It must in this format >(num)%. ie. >10%\n"
            "Assay_information.yaml: "
            "Please double check your alteration_types column.  "
            "This column must only be these values: snv, small_indels, "
            "gene_level_cna, intragenic_cna, structural_variants\n"
            "Assay_information.yaml: "
            "Please double check your preservation_technique column.  "
            "This column must only be these values: FFPE, fresh_frozen, NA\n"
            "Assay_information.yaml: "
            "Please double check your coverage column.  "
            "This column must only be these values: "
            "hotspot_regions, coding_exons, introns, promoters\n"
        )

        patch_get_gdc.called_once_with("read_group")
        assert error == expected_errors
        assert warning == ""



import os

import pytest

from genie import create_case_lists

study_id = "test"

clinical_file_map = {"": ["FOOBAR", "NEED"]}
clinical_file_map["Testing2"] = ["test1"]
clinical_file_map["Test1 Now, Please/foo"] = ["wow"]

sequenced_case_list_files = create_case_lists.write_case_list_sequenced(
    ["test1", "test2"], "./", study_id
)
case_list_cna_path = create_case_lists.write_case_list_cna(
    ["test1", "test2"], "./", study_id
)
case_list_cnaseq_path = create_case_lists.write_case_list_cnaseq(
    ["test1", "test2"], "./", study_id
)


def test_filenames_write_case_list_files():
    case_list_files = create_case_lists.write_case_list_files(
        clinical_file_map, "./", study_id
    )
    required_files = [
        "cases_Test1_Now_Please_foo.txt",
        "cases_no_oncotree_code.txt",
        "cases_Testing2.txt",
    ]
    basenames = [os.path.basename(case_file) for case_file in case_list_files]

    assert all([req_file in basenames for req_file in required_files])


expected_change_text = (
    "cancer_study_identifier: test\n"
    "stable_id: test_Test1_Now_Please_foo\n"
    "case_list_name: Tumor Type: Test1 Now, Please/foo\n"
    "case_list_description: All tumors with cancer type "
    "Test1 Now, Please/foo\n"
    "case_list_ids: wow"
)

expected_same_text = (
    "cancer_study_identifier: test\n"
    "stable_id: test_Testing2\n"
    "case_list_name: Tumor Type: Testing2\n"
    "case_list_description: All tumors with cancer type Testing2\n"
    "case_list_ids: test1"
)

expected_nocode_text = (
    "cancer_study_identifier: test\n"
    "stable_id: test_no_oncotree_code\n"
    "case_list_name: Tumor Type: NA\n"
    "case_list_description: All tumors with cancer type NA\n"
    "case_list_ids: FOOBAR\tNEED"
)


@pytest.fixture(
    params=[
        # tuple with (input, expectedOutput)
        ("", clinical_file_map[""], expected_nocode_text),
        ("Testing2", clinical_file_map["Testing2"], expected_same_text),
        (
            "Test1 Now, Please/foo",
            clinical_file_map["Test1 Now, Please/foo"],
            expected_change_text,
        ),
    ]
)
def oncotree_write_params(request):
    return request.param


def test__write_single_oncotree_case_list(oncotree_write_params):
    (cancer_type, ids, expected_text) = oncotree_write_params
    caselist_path = create_case_lists._write_single_oncotree_case_list(
        cancer_type, ids, study_id, "./"
    )
    with open(caselist_path, "r") as case_list:
        caselist_text = case_list.read()
    assert caselist_text == expected_text
    os.remove(caselist_path)


def test_filenames_write_case_list_sequenced():
    first = os.path.basename(sequenced_case_list_files[0])
    assert first == "cases_sequenced.txt"
    second = os.path.basename(sequenced_case_list_files[1])
    assert second == "cases_all.txt"


def test_sequencetext_write_case_list_sequenced():
    expected_text = (
        "cancer_study_identifier: test\n"
        "stable_id: test_sequenced\n"
        "case_list_name: Sequenced Tumors\n"
        "case_list_description: All sequenced samples\n"
        "case_list_ids: test1\ttest2"
    )
    with open(sequenced_case_list_files[0], "r") as case_list:
        caselist_text = case_list.read()
    assert caselist_text == expected_text
    os.remove(sequenced_case_list_files[0])


def test_all_write_case_list_sequenced():
    expected_text = (
        "cancer_study_identifier: test\n"
        "stable_id: test_all\n"
        "case_list_name: All samples\n"
        "case_list_description: All samples\n"
        "case_list_ids: test1\ttest2"
    )
    with open(sequenced_case_list_files[1], "r") as case_list:
        caselist_text = case_list.read()
    assert caselist_text == expected_text
    os.remove(sequenced_case_list_files[1])


def test_filename_write_case_list_cna():
    assert os.path.basename(case_list_cna_path) == "cases_cna.txt"


def test_filename_write_case_list_cnaseq():
    assert os.path.basename(case_list_cnaseq_path) == "cases_cnaseq.txt"


def test_cnatext_write_case_list_cna():
    expected_text = (
        "cancer_study_identifier: test\n"
        "stable_id: test_cna\n"
        "case_list_name: Samples with CNA\n"
        "case_list_description: Samples with CNA\n"
        "case_list_ids: test1\ttest2"
    )
    with open(case_list_cna_path, "r") as case_list:
        caselist_text = case_list.read()
    assert caselist_text == expected_text
    os.remove(case_list_cna_path)


def test_cnaseq_write_case_list_cnaseq():
    expected_text = (
        "cancer_study_identifier: test\n"
        "stable_id: test_cnaseq\n"
        "case_list_name: Samples with CNA and mutation\n"
        "case_list_description: Samples with CNA and mutation\n"
        "case_list_ids: test1\ttest2"
    )
    with open(case_list_cnaseq_path, "r") as case_list:
        caselist_text = case_list.read()
    assert caselist_text == expected_text
    os.remove(case_list_cnaseq_path)



"""Test GENIE Bed class"""
import tempfile
import shutil
from unittest import mock
from unittest.mock import patch

import pandas as pd
from pandas.testing import assert_frame_equal
import pytest
import synapseclient

import genie_registry.bed
from genie_registry.bed import bed
from genie_registry.bedSP import bedSP

if not shutil.which("bedtools"):
    pytest.skip("bedtools is not found, skipping bed tests", allow_module_level=True)

GENE_GTF_TEXT = '2\tprotein_coding\tgene\t69688532\t69901481\t.\t-\t.\tgene_id "ENSG00000115977"; gene_name "AAK1"; gene_source "ensembl_havana"; gene_biotype "protein_coding";\n9\tprotein_coding\tgene\t99401859\t99417585\t.\t-\t.\tgene_id "ENSG00000158122"; gene_name "AAED1"; gene_source "ensembl_havana"; gene_biotype "protein_coding";\n12\tprotein_coding\tgene\t53701240\t53718648\t.\t-\t.\tgene_id "ENSG00000094914"; gene_name "AAAS"; gene_source "ensembl_havana"; gene_biotype "protein_coding";\n19\tprotein_coding\tgene\t44047192\t44084625\t.\t-\t.\tgene_id "ENSG00000073050"; gene_name "XRCC1"; gene_source "ensembl_havana"; gene_biotype "protein_coding";\n19\tprotein_coding\tgene\t44080952\t44088116\t.\t+\t.\tgene_id "ENSG00000234465"; gene_name "PINLYP"; gene_source "ensembl_havana"; gene_biotype "protein_coding";'
EXON_GTF_TEXT = '2\tprocessed_transcript\texon\t69688432\t69689532\t.\t-\t.\tgene_id "ENSG00000115977"; transcript_id "ENST00000492192"; exon_number "2"; gene_name "AAK1"; gene_source "ensembl_havana"; gene_biotype "protein_coding"; transcript_name "AAK1-009"; transcript_source "havana"; exon_id "ENSE00001882560";\n9\tprotein_coding\texon\t99416987\t99417030\t.\t-\t.\tgene_id "ENSG00000158122"; transcript_id "ENST00000411939"; exon_number "1"; gene_name "AAED1"; gene_source "ensembl_havana"; gene_biotype "protein_coding"; transcript_name "AAED1-003"; transcript_source "havana"; exon_id "ENSE00001768346"; tag "cds_start_NF"; tag "mRNA_start_NF";\n12\tretained_intron\texon\t53702509\t53702599\t.\t-\t.\tgene_id "ENSG00000094914"; transcript_id "ENST00000550033"; exon_number "4"; gene_name "AAAS"; gene_source "ensembl_havana"; gene_biotype "protein_coding"; transcript_name "AAAS-019"; transcript_source "havana"; exon_id "ENSE00003694270";\n19\tprotein_coding\texon\t44084517\t44084625\t.\t-\t.\tgene_id "ENSG00000073050"; transcript_id "ENST00000598165"; exon_number "1"; gene_name "XRCC1"; gene_source "ensembl_havana"; gene_biotype "protein_coding"; transcript_name "XRCC1-008"; transcript_source "havana"; exon_id "ENSE00003137784"; tag "cds_end_NF"; tag "mRNA_end_NF";\n19\tprotein_coding\texon\t44084696\t44084739\t.\t+\t.\tgene_id "ENSG00000234465"; transcript_id "ENST00000562255"; exon_number "1"; gene_name "PINLYP"; gene_source "ensembl_havana"; gene_biotype "protein_coding"; transcript_name "PINLYP-001"; transcript_source "havana"; tag "CCDS"; ccds_id "CCDS58667"; exon_id "ENSE00002599477";'
GENE_TEMP = tempfile.NamedTemporaryFile()
EXON_TEMP = tempfile.NamedTemporaryFile()

with open(GENE_TEMP.name, "w") as gene:
    gene.write(GENE_GTF_TEXT)
with open(EXON_TEMP.name, "w") as exon:
    exon.write(EXON_GTF_TEXT)


def create_mock_table(dataframe):
    table = mock.create_autospec(synapseclient.table.CsvFileTable)
    table.asDataFrame.return_value = dataframe
    return table


def table_query_results(*args):
    return table_query_results_map[args]


symbols = pd.DataFrame(
    dict(
        hgnc_symbol=["AAK1", "AAED1", "AAAS", "PINLYP", "XRCC1"],
        chromosome_name=["2", "9", "12", "19", "19"],
        start_position=[69688532, 99401859, 53701240, 44080952, 44047192],
        end_position=[69901481, 99417585, 53718648, 44088116, 44084625],
    )
)

# This is the gene positions that all bed dataframe will be processed against
table_query_results_map = {
    ("SELECT * FROM syn11806563",): create_mock_table(symbols),
}

syn = mock.create_autospec(synapseclient.Synapse)
syn.tableQuery.side_effect = table_query_results

seq_assay_id = "SAGE-Test"
new_path = "new.bed"
parentid = "synTest"
bed_class = bed(syn, "SAGE")
bedsp_class = bedSP(syn, "SAGE")


def test_perfect___process():
    """Process perfect bed file"""
    expected_beddf = pd.DataFrame(
        dict(
            Chromosome=["2", "9", "12", "19", "19"],
            Start_Position=[69688533, 99401860, 53701241, 44084466, 44084466],
            End_Position=[69901480, 99417584, 53718647, 44084638, 44084638],
            Hugo_Symbol=["AAK1", "AAED1", "AAAS", "XRCC1", "PINLYP"],
            includeInPanel=[True, True, True, True, True],
            clinicalReported=[True, True, False, False, True],
            ID=["AAK1", "AAED1", "AAAS", "XRCC1", "foo"],
            SEQ_ASSAY_ID=[
                "SAGE-TEST",
                "SAGE-TEST",
                "SAGE-TEST",
                "SAGE-TEST",
                "SAGE-TEST",
            ],
            Feature_Type=["exon", "exon", "exon", "exon", "exon"],
            CENTER=["SAGE", "SAGE", "SAGE", "SAGE", "SAGE"],
        )
    )

    expected_beddf.sort_values("ID", inplace=True)
    expected_beddf.reset_index(drop=True, inplace=True)
    beddf = pd.DataFrame(
        {
            0: ["2", "9", "12", "19", "19"],
            1: [69688533, 99401860, 53701241, 44084466, 44084466],
            2: [69901480, 99417584, 53718647, 44084638, 44084638],
            3: ["AAK1", "AAED1", "AAAS", "XRCC1", "foo"],
            4: [True, True, True, 1, 1],
            5: [True, True, False, 0, 1],
        }
    )
    with patch.object(
        genie_registry.bed, "create_gtf", return_value=(EXON_TEMP.name, GENE_TEMP.name)
    ):
        new_beddf = bed_class._process(
            beddf, seq_assay_id, new_path, parentid, create_panel=False
        )
        new_beddf.sort_values("ID", inplace=True)
        new_beddf.reset_index(drop=True, inplace=True)
        assert_frame_equal(
            expected_beddf, new_beddf[expected_beddf.columns], check_dtype=False
        )


def test_includeinpanel___process():
    """
    Make sure includeInPanel column is propogated and
    intergenic region is captured
    """

    expected_beddf = pd.DataFrame(
        dict(
            Chromosome=["2", "9", "12", "19"],
            Start_Position=[69688432, 1111, 53700240, 44080953],
            End_Position=[69689532, 1111, 53719548, 44084624],
            Hugo_Symbol=["AAK1", float("nan"), "AAAS", float("nan")],
            includeInPanel=[True, True, False, True],
            clinicalReported=[float("nan")] * 4,
            ID=["foo", "bar", "baz", "boo"],
            SEQ_ASSAY_ID=["SAGE-TEST", "SAGE-TEST", "SAGE-TEST", "SAGE-TEST"],
            Feature_Type=["exon", "intergenic", "exon", "exon"],
            CENTER=["SAGE", "SAGE", "SAGE", "SAGE"],
        )
    )

    expected_beddf.sort_values("Chromosome", inplace=True)
    expected_beddf.reset_index(drop=True, inplace=True)

    # symbols that can't be map should be null,
    # includeInPanel column should be included if it exists
    beddf = pd.DataFrame(
        {
            0: ["2", "9", "12", "19"],
            1: [69688432, 1111, 53700240, 44080953],
            2: [69689532, 1111, 53719548, 44084624],
            3: ["foo", "bar", "baz", "boo"],
            4: [True, True, 0, 1],
        }
    )
    with patch.object(
        genie_registry.bed, "create_gtf", return_value=(EXON_TEMP.name, GENE_TEMP.name)
    ):
        new_beddf = bedsp_class._process(
            beddf, seq_assay_id, new_path, parentid, create_panel=False
        )
        new_beddf.sort_values("Chromosome", inplace=True)
        new_beddf.reset_index(drop=True, inplace=True)
        assert_frame_equal(
            expected_beddf, new_beddf[expected_beddf.columns], check_dtype=False
        )


def test_clinicalreport___process():
    expected_beddf = pd.DataFrame(
        dict(
            Chromosome=["2", "9", "12", "19"],
            Start_Position=[69688432, 1111, 53700240, 44080953],
            End_Position=[69689532, 1111, 53719548, 44084624],
            Hugo_Symbol=["AAK1", float("nan"), "AAAS", float("nan")],
            includeInPanel=[True, True, False, True],
            clinicalReported=[float("nan")] * 4,
            ID=["foo", "bar", "baz", "boo"],
            SEQ_ASSAY_ID=["SAGE-TEST", "SAGE-TEST", "SAGE-TEST", "SAGE-TEST"],
            Feature_Type=["exon", "intergenic", "exon", "exon"],
            CENTER=["SAGE", "SAGE", "SAGE", "SAGE"],
        )
    )

    expected_beddf.sort_values("Chromosome", inplace=True)
    expected_beddf.reset_index(drop=True, inplace=True)

    # symbols that can't be map should be null,
    # includeInPanel column should be included if it exists
    beddf = pd.DataFrame(
        {
            0: ["2", "9", "12", "19"],
            1: [69688432, 1111, 53700240, 44080953],
            2: [69689532, 1111, 53719548, 44084624],
            3: ["foo", "bar", "baz", "boo"],
            4: [True, True, False, True],
            5: [True, float("nan"), False, True],
        }
    )
    with patch.object(
        genie_registry.bed, "create_gtf", return_value=(EXON_TEMP.name, GENE_TEMP.name)
    ):
        new_beddf = bedsp_class._process(
            beddf, seq_assay_id, new_path, parentid, create_panel=False
        )
        new_beddf.sort_values("Chromosome", inplace=True)
        new_beddf.reset_index(drop=True, inplace=True)
        assert_frame_equal(
            expected_beddf, new_beddf[expected_beddf.columns], check_dtype=False
        )


def test_filetype():
    assert bed_class._fileType == "bed"
    assert bedsp_class._fileType == "bedSP"


@pytest.fixture(
    params=[(["foo"]), (["SAGE-test.txt"]), (["foo"]), (["nonGENIE_SAGE-test.txt"])]
)
def filename_fileformat_map(request):
    return request.param


def test_incorrect_validatefilename(filename_fileformat_map):
    filepath_list = filename_fileformat_map
    with pytest.raises(AssertionError):
        bed_class.validateFilename(filepath_list)


def test_correct_validatefilename():
    assert bed_class.validateFilename(["SAGE-test.bed"]) == "bed"
    assert bedsp_class.validateFilename(["nonGENIE_SAGE-test.bed"]) == "bedSP"


def test_perfect__validate():
    bedDf = pd.DataFrame(
        dict(
            a=["2", "9", "12"],
            b=[69688533, 99401860, 53701241],
            c=[69901480, 99417584, 53718647],
            d=["AAK1", "AAED1", "AAAS"],
            e=[True, True, False],
            f=[True, True, False],
        )
    )

    error, warning = bed_class._validate(bedDf)
    assert error == ""
    assert warning == ""


def test_90percent_boundary__validate():
    bedDf = pd.DataFrame(
        dict(
            a=["2", "9", "12"],
            b=[69688432, 99416585, 53700240],
            c=[69689532, 99417685, 53719548],
            d=["AAK1", "AAED1", "AAAS"],
            e=[True, True, False],
            f=[True, True, False],
        )
    )
    error, warning = bed_class._validate(bedDf)
    assert error == ""
    assert warning == ""


def test_missingcols_failure__validate():
    emptydf = pd.DataFrame()
    error, warning = bed_class._validate(emptydf)
    expected_errors = (
        "BED file: Must at least have five columns in this order: "
        "Chromosome, Start_Position, End_Position, Hugo_Symbol, "
        "includeInPanel. Make sure there are no headers.\n"
    )
    assert error == expected_errors
    assert warning == ""


def test_hugosymbol_failure__validate():
    bedDf = pd.DataFrame(
        dict(
            a=["2", "9", "12"],
            b=[69688533, 99401860, 53701241],
            c=[69901480, 99417584, 53718647],
            d=["+", float("nan"), "AAAS"],
            e=[True, True, False],
        )
    )
    error, warning = bed_class._validate(bedDf)
    expected_errors = (
        "BED file: You cannot submit any null symbols.\n"
        "BED file: Fourth column must be the Hugo_Symbol column, "
        "not the strand column\n"
    )
    assert error == expected_errors
    assert warning == ""


def test_badinputs_failure__validate():
    bedDf = pd.DataFrame(
        dict(
            a=["2", "9", "12"],
            b=["69688533", 99401860, 53701241],
            c=[69901480, "99417584", 53718647],
            d=["AAK1", "AAED1", "AAAS"],
            e=[True, False, "foobar"],
        )
    )

    error, warning = bed_class._validate(bedDf)
    expected_errors = (
        "BED file: The Start_Position column must only be integers. "
        "Make sure there are no headers.\n"
        "BED file: The End_Position column must only be integers. "
        "Make sure there are no headers.\n"
        "BED file: Please double check your includeInPanel column.  "
        "This column must only be these values: True, False\n"
    )
    assert error == expected_errors
    assert warning == ""


def test_90percentboundary_failure__validate():
    # Test 90% boundary failure boundary, with incorrect gene names
    bedDf = pd.DataFrame(
        dict(
            a=["2", "9", "12"],
            b=[69901381, 4345, 11111],
            c=[69911481, 99417590, 11113],
            d=["foo", "foo", "AAAS"],
            e=[True, True, False],
        )
    )

    error, warning = bedsp_class._validate(bedDf)
    expected_errors = (
        "BED file: You have no correct gene symbols. "
        "Make sure your gene symbol column (4th column) is formatted like so: "
        "SYMBOL(;optionaltext).  Optional text can be semi-colon separated.\n"
    )
    expected_warnings = (
        "BED file: Any gene names that can't be remapped will be null.\n"
    )
    assert error == expected_errors
    assert warning == expected_warnings


def test_overlapping__validate():
    """
    Check if genes that overlap have no errors
    - Submitted bed start position in a gene
    - Submitted bed end position in a gene
    - Submitted bed region surrounds a gene
    """
    beddf = pd.DataFrame(
        dict(
            a=["2", "9", "2"],
            b=[1111, 4345, 69901281],
            c=[69880186, 99417590, 70000000],
            d=["AAK1", "AAED1", "AAK1"],
            e=[True, False, True],
        )
    )
    error, warning = bedsp_class._validate(beddf)
    assert error == ""
    assert warning == ""


def test_symbolnull_failure__validate():
    # Test 2 gene symbols returned NULL
    bedDf = pd.DataFrame(
        dict(a=["19"], b=[44080953], c=[44084624], d=["AAK1"], e=[False])
    )

    error, warning = bedsp_class._validate(bedDf)
    expected_errors = (
        "BED file: You have no correct gene symbols. "
        "Make sure your gene symbol column (4th column) is formatted like so: "
        "SYMBOL(;optionaltext).  Optional text can be semi-colon separated.\n"
    )
    expected_warnings = (
        "BED file: Any gene names that can't be remapped will be null.\n"
    )
    assert error == expected_errors
    assert warning == expected_warnings



# import synapseclient
# import pandas as pd
# import mock
# from nose.tools import assert_raises
# import os
# import sys

# from genie.cbs import cbs

# def test_processing():

#   syn = mock.create_autospec(synapseclient.Synapse)

#   cbsClass = cbs(cbs, "SAGE")
#   pass

# def test_validation():

#   syn = mock.create_autospec(synapseclient.Synapse)

#   cbsClass = cbs(cbs, "SAGE")

#   assert_raises(AssertionError, cbsClass.validateFilename, ["foo"])
#   assert_raises(AssertionError, cbsClass.validateFilename, ["GENIE-test.txt"])
#   assert cbsClass.validateFilename(["GENIE-SAGE-test.cbs"]) == "cbs"

#   cbsDf = pd.DataFrame({"ID":['ID1','ID2','ID3','ID4','ID5'],
#                         "Chr":[1,2,3,4,5],
#                         "Start":[1,2,3,4,3],
#                         "End":[1,2,3,4,3],
#                         "Probes":[1,2,3,4,3],
#                         "Log2Ratio":[1,2,3,4,3]})

#   error, warning = cbsClass._validate(cbsDf)
#   assert error == ""
#   assert warning == ""

#   cbsDf = pd.DataFrame({"ID":['ID1','ID2','ID3','ID4','ID5'],
#                         "Chr":[1,2,float('nan'),4,5],
#                         "Start":[1,2,3,4,float('nan')],
#                         "End":[1,2,3,4,3],
#                         "Log2Ratio":[1,2,3,4,3]})
#   expectedErrors = ("Your cbs file must at least have these headers: Probes.\n"
#                     "cbs: No null or empty values allowed in column(s): Chr, Start.\n")
#   error, warning = cbsClass._validate(cbsDf)
#   assert error == expectedErrors
#   assert warning == ""

#   cbsDf = pd.DataFrame({"ID":['ID1','ID2','ID3','ID4','ID5'],
#                         "Chr":[1,2,3,4,5],
#                         "Start":[1,2,3,4.3,3],
#                         "End":[1,2,3.4,4,3],
#                         "Probes":[1,2,3,33.3,3],
#                         "Log2Ratio":[1,2,'f.d',4,3]})
#   error, warning = cbsClass._validate(cbsDf)
#   expectedErrors = ("cbs: Only integars allowed in these column(s): Start, End, Probes.\n"
#                     "cbs: Only numerical values allowed in Log2Ratio.\n")
#   assert error == expectedErrors
#   assert warning == ""
